#!/usr/bin/env python
# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****

import copy
import getpass
import os
import re
import signal
import subprocess
import sys
import time
import socket

# load modules from parent dir
sys.path.insert(1, os.path.dirname(sys.path[0]))

from mozharness.mozilla.buildbot import TBPL_SUCCESS, BuildbotMixin
from mozharness.base.errors import BaseErrorList
from mozharness.base.log import INFO, ERROR, FATAL
from mozharness.base.vcs.vcsbase import MercurialScript
from mozharness.mozilla.blob_upload import BlobUploadMixin, blobupload_config_options
from mozharness.mozilla.mozbase import MozbaseMixin
from mozharness.mozilla.testing.mozpool import MozpoolMixin
from mozharness.mozilla.testing.device import SUTDeviceMozdeviceMixin
from mozharness.mozilla.testing.testbase import TestingMixin, testing_config_options

SUITE_CATEGORIES = ['mochitest', 'reftest', 'crashtest', 'jsreftest', 'robocop', 'instrumentation', 'xpcshell', 'jittest', 'cppunittest']


class PandaTest(TestingMixin, MercurialScript, BlobUploadMixin, MozpoolMixin, BuildbotMixin, SUTDeviceMozdeviceMixin, MozbaseMixin):
    test_suites = SUITE_CATEGORIES
    config_options = [
        [["--mozpool-api-url"], {
            "dest": "mozpool_api_url",
            "help": "Override mozpool api url",
        }],
        [["--mozpool-device"], {
            "dest": "mozpool_device",
            "help": "Set Panda device to run tests on",
        }],
        [["--mozpool-assignee"], {
            "dest": "mozpool_assignee",
            "help": "Set mozpool assignee (requestor name, free-form)",
        }],
        [["--total-chunks"], {
            "action": "store",
            "dest": "total_chunks",
            "help": "Number of total chunks",
        }],
        [["--this-chunk"], {
            "action": "store",
            "dest": "this_chunk",
            "help": "Number of this chunk",
        }],
        [["--extra-args"], {
            "action": "store",
            "dest": "extra_args",
            "help": "Extra arguments",
        }],
        [['--mochitest-suite', ], {
            "action": "extend",
            "dest": "specified_mochitest_suites",
            "type": "string",
            "help": "Specify which mochi suite to run. "
                    "Suites are defined in the config file.\n"
                    "Examples: 'all', 'plain1', 'plain5', 'chrome', or 'a11y'"}
         ],
        [['--reftest-suite', ], {
            "action": "extend",
            "dest": "specified_reftest_suites",
            "type": "string",
            "help": "Specify which reftest suite to run. "
                    "Suites are defined in the config file.\n"
                    "Examples: 'all', 'crashplan', or 'jsreftest'"}
         ],
        [['--crashtest-suite', ], {
            "action": "extend",
            "dest": "specified_crashtest_suites",
            "type": "string",
            "help": "Specify which crashtest suite to run. "
                    "Suites are defined in the config file\n."
                    "Examples: 'crashtest'"}
         ],
        [['--jsreftest-suite', ], {
            "action": "extend",
            "dest": "specified_jsreftest_suites",
            "type": "string",
            "help": "Specify which jsreftest suite to run. "
                    "Suites are defined in the config file\n."
                    "Examples: 'jsreftest'"}
         ],
        [['--robocop-suite', ], {
            "action": "extend",
            "dest": "specified_robocop_suites",
            "type": "string",
            "help": "Specify which robocop suite to run. "
                    "Suites are defined in the config file\n."
                    "Examples: 'robocop'"}
         ],
        [['--instrumentation-suite', ], {
            "action": "extend",
            "dest": "specified_instrumentation_suites",
            "type": "string",
            "help": "Specify which instrumentation suite to run. "
                    "Suites are defined in the config file\n."
                    "Examples: 'browser', 'background'"}
         ],
         [['--xpcshell-suite', ], {
            "action": "extend",
            "dest": "specified_xpcshell_suites",
            "type": "string",
            "help": "Specify which xpcshell suite to run. "
                    "Suites are defined in the config file\n."
                    "Examples: 'xpcshell'"}
         ],
         [['--jittest-suite', ], {
            "action": "extend",
            "dest": "specified_jittest_suites",
            "type": "string",
            "help": "Specify which jittest suite to run. "
                    "Suites are defined in the config file\n."
                    "Examples: 'jittest'"}
         ],
        [['--cppunittest-suite', ], {
            "action": "extend",
            "dest": "specified_cppunittest_suites",
            "type": "string",
            "help": "Specify which cpp unittest suite to run. "
                    "Suites are defined in the config file\n."
                    "Examples: 'cppunittest'"}
         ],
        [['--run-all-suites', ], {
            "action": "store_true",
            "dest": "run_all_suites",
            "default": False,
            "help": "This will run all suites that are specified "
                    "in the config file. You do not need to specify "
                    "any other suites. Beware, this may take a while ;)"}
         ],
    ] + copy.deepcopy(testing_config_options) + \
        copy.deepcopy(blobupload_config_options)

    error_list = []
    mozpool_handler = None

    virtualenv_modules = [
        'mozpoolclient',
    ]

    def __init__(self, require_config_file=False):
        super(PandaTest, self).__init__(
            config_options=self.config_options,
            all_actions=['clobber',
                         'read-buildbot-config',
                         'download-and-extract',
                         'create-virtualenv',
                         'request-device',
                         'run-test',
                         'close-request'],
            default_actions=['clobber',
                             'read-buildbot-config',
                             'download-and-extract',
                             'create-virtualenv',
                             'request-device',
                             'run-test',
                             'close-request'],
            require_config_file=require_config_file,
            config={'virtualenv_modules': self.virtualenv_modules})

        self.mozpool_assignee = self.config.get('mozpool_assignee', getpass.getuser())
        self.request_url = None
        self.installer_url = self.config.get("installer_url")
        self.test_url = self.config.get("test_url")
        self.mozpool_device = self.config.get("mozpool_device")
        self.symbols_url = self.config.get('symbols_url')

    def postflight_read_buildbot_config(self):
        super(PandaTest, self).postflight_read_buildbot_config()
        self.mozpool_device = self.config.get('mozpool_device', self.buildbot_config.get('properties')["slavename"])
        dirs = self.query_abs_dirs()
        #touch the shutdown file
        shutdown_file = os.path.join(dirs['shutdown_dir'], 'shutdown.stamp')
        try:
            self.info("*** Touching the shutdown file **")
            open(shutdown_file, 'w').close()
        except Exception, e:
            self.warning("We failed to create the shutdown file: str(%s)" % str(e))

    def request_device(self):
        self.retrieve_android_device(b2gbase="")
        env = self.query_env()
        cmd = [self.query_exe('python'), self.config.get("verify_path")]
        if self.run_command(cmd, env=env):
            self.critical("Preparing to abort run due to failed verify check.")
            self.close_request()
            self.fatal("Dying due to failing verification")
        else:
            self.info("Successfully verified the device")

    def _sut_prep_steps(self):
        device_time = self.set_device_epoch_time()
        self.info("Current time on device: %s - %s" %
                  (device_time, time.strftime("%x %H:%M:%S", time.gmtime(float(device_time)))))

    def download_and_extract(self):
        """
        Provides the target suite categories to TestingMixin.download_
        """
        if self.config.get('run_all_suites'):
            target_categories = SUITE_CATEGORIES
        else:
            target_categories = [cat for cat in SUITE_CATEGORIES
                                 if self._query_specified_suites(cat) is not None]
        super(PandaTest, self).download_and_extract(suite_categories=target_categories)

    def _query_try_flavor(self, category, suite):
        flavors = {
            "mochitest": [("plain.*", "mochitest"),
                          ("browser-chrome.*", "browser-chrome"),
                          ("mochitest-devtools-chrome.*", "devtools-chrome"),
                          ("chrome", "chrome")],
            "xpcshell": [("xpcshell", "xpcshell")],
            "reftest": [("reftest", "reftest"),
                        ("crashtest", "crashtest")]
        }
        for suite_pattern, flavor in flavors.get(category, []):
            if re.compile(suite_pattern).match(suite):
                return flavor

    def _run_category_suites(self, suite_category, preflight_run_method=None):
        """run suite(s) to a specific category"""

        env = self.query_env(partial_env={'DM_TRANS': "sut", 'TEST_DEVICE': self.mozpool_device})
        self.info("Running tests...")

        suites = self._query_specified_suites(suite_category)
        level = INFO

        if preflight_run_method:
            preflight_run_method(suites)
        if suites:
            self.info('#### Running %s suites' % suite_category)
            for suite in suites:
                dirs = self.query_abs_dirs()
                self._download_unzip_hostutils()
                abs_base_cmd = self._query_abs_base_cmd(suite_category, suite)

                should_install_app = True
                if 'cppunittest' in suite:
                    should_install_app = False
                if 'robocop' in suite:
                    self._download_robocop_apk()
                if 'jittest' in suite:
                    should_install_app = False

                if should_install_app:
                    self._install_app()
                cmd = abs_base_cmd[:]

                flavor = self._query_try_flavor(suite_category, suite)
                try_options, try_tests = self.try_args(flavor)

                cmd.extend(self.query_options(suites[suite],
                                              try_options))
                cmd.extend(self.query_tests_args(try_tests))

                tests = self.config["suite_definitions"][suite_category].get("tests", [])
                cmd += tests

                tbpl_status, log_level = None, None
                error_list = BaseErrorList + [{
                    'regex': re.compile(r"(?:TEST-UNEXPECTED-FAIL|PROCESS-CRASH) \| .* \| (application crashed|missing output line for total leaks!|negative leaks caught!|\d+ bytes leaked)"),
                    'level': ERROR,
                }]
                c = self.config
                if c.get('minidump_stackwalk_path'):
                    env['MINIDUMP_STACKWALK'] = c['minidump_stackwalk_path']
                env['MOZ_UPLOAD_DIR'] = self.query_abs_dirs()['abs_blob_upload_dir']
                env['MINIDUMP_SAVE_PATH'] = self.query_abs_dirs()['abs_blob_upload_dir']

                env = self.query_env(partial_env=env, log_level=INFO)
                if env.has_key('PYTHONPATH'):
                    del env['PYTHONPATH']

                parser = self.get_test_output_parser(suite_category,
                                                     config=self.config,
                                                     error_list=error_list,
                                                     log_obj=self.log_obj)

                return_code = self.run_command(cmd,
                                               cwd=dirs['abs_test_install_dir'],
                                               env=env,
                                               output_parser=parser)

                tbpl_status, log_level = parser.evaluate_parser(return_code)

                if tbpl_status != TBPL_SUCCESS:
                    self.info("Output logcat...")
                    try:
                        lines = self.get_logcat()
                        self.info("*** STARTING LOGCAT ***")
                        for l in lines:
                            self.info(l)
                        self.info("*** END LOGCAT ***")
                    except Exception, e:
                        self.warning("We failed to run logcat: str(%s)" % str(e))

                parser.append_tinderboxprint_line(suite)
                self.buildbot_status(tbpl_status, level=level)

                self.log("The %s suite: %s ran with return status: %s" %
                         (suite_category, suite, tbpl_status), level=log_level)

    def _query_specified_suites(self, category):
        # logic goes: if at least one '--{category}-suite' was given,
        # then run only that(those) given suite(s). Elif no suites were
        # specified and the --run-all-suites flag was given,
        # run all {category} suites. Anything else, run no suites.
        c = self.config
        all_suites = c.get('all_%s_suites' % (category))
        specified_suites = c.get('specified_%s_suites' % (category))  # list

        suites = None

        if specified_suites:
            if 'all' in specified_suites:
                # useful if you want a quick way of saying run all suites
                # of a specific category.
                suites = all_suites
            else:
                # suites gets a dict of everything from all_suites where a key
                # is also in specified_suites
                suites = dict((key, all_suites.get(key)) for key in
                              specified_suites if key in all_suites.keys())
        else:
            if c.get('run_all_suites'):  # needed if you dont specify any suites
                suites = all_suites

        return suites

    def run_test(self):
       # do we need to set the device time? command doesn't work anyways
       # self._sut_prep_steps()
        env = self.query_env()
        env["DM_TRANS"] = "sut"
        env["TEST_DEVICE"] = self.mozpool_device
        self.mkdir_p(self.abs_dirs['abs_blob_upload_dir'])

        self._start_logcat()

        self.info("Running tests...")

        for category in SUITE_CATEGORIES:
            self._run_category_suites(category)

        self._stop_logcat()

    def _start_logcat(self):
        # Start logcat.py as a separate process continuously pulling logcat from
        # the device and writing to a file. Output is written directly to
        # the blobber upload directory so that it is uploaded automatically
        # at the end of the job.
        device_ip = socket.gethostbyname(self.mozpool_device)
        logcat_path = os.path.join(self.abs_dirs['abs_blob_upload_dir'], 'logcat.log')
        logcat_cmd = ['python', '-u', self.config.get("logcat_path"), \
            device_ip, logcat_path, '-v time']
        self.info('Starting logcat: %s' % str(logcat_cmd))
        self.logcat_proc = subprocess.Popen(logcat_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    def _stop_logcat(self):
        # Signal logcat.py so that it can cleanup (kill the device logcat process)
        self.logcat_proc.send_signal(signal.SIGINT)
        self.logcat_proc.kill()
        out, err = self.logcat_proc.communicate()
        self.info("logcat.py output:\n%s\n%s\n" % (out, err))

    def _download_unzip_hostutils(self):
        c = self.config
        dirs = self.query_abs_dirs()
        self.host_utils_url = c['hostutils_url']
        # get the zip and extract it
        self.download_unzip(self.host_utils_url, dirs['abs_hostutils_dir'])

    def _install_app(self):
        c = self.config
        base_work_dir = c['base_work_dir']
        cmd = ['python', self.config.get("install_app_path"), self.device_ip, 'build/' + str(self.filename_apk), self.app_name]
        self.run_command(cmd, cwd=base_work_dir, halt_on_failure=True, fatal_exit_code=3)

    def _download_robocop_apk(self):
        dirs = self.query_abs_dirs()
        self.apk_url = self.installer_url[:self.installer_url.rfind('/')]
        robocop_url = self.apk_url + '/robocop.apk'
        self.info("Downloading robocop...")
        self.download_file(robocop_url, 'robocop.apk', dirs['abs_work_dir'], error_level=FATAL)

    def query_abs_dirs(self):
        if self.abs_dirs:
            return self.abs_dirs
        abs_dirs = super(PandaTest, self).query_abs_dirs()
        dirs = {}
        dirs['abs_test_install_dir'] = os.path.join(
            abs_dirs['abs_work_dir'], 'tests')
        dirs['abs_test_bin_dir'] = os.path.join(dirs['abs_test_install_dir'], 'bin')
        dirs['abs_mochitest_dir'] = os.path.join(
            dirs['abs_test_install_dir'], 'mochitest')
        dirs['abs_reftest_dir'] = os.path.join(
            dirs['abs_test_install_dir'], 'reftest')
        dirs['abs_crashtest_dir'] = os.path.join(
            dirs['abs_test_install_dir'], 'reftest')
        dirs['abs_jsreftest_dir'] = os.path.join(
            dirs['abs_test_install_dir'], 'reftest')
        dirs['abs_xpcshell_dir'] = os.path.join(
            dirs['abs_test_install_dir'], 'xpcshell')
        dirs['abs_xre_dir'] = os.path.join(
            abs_dirs['abs_work_dir'], 'xre')
        dirs['abs_utility_path'] = os.path.join(
            abs_dirs['abs_work_dir'], 'bin')
        dirs['abs_certificate_path'] = os.path.join(
            abs_dirs['abs_work_dir'], 'certs')
        dirs['abs_hostutils_dir'] = os.path.join(
            abs_dirs['abs_work_dir'], 'hostutils')
        dirs['abs_robocop_dir'] = os.path.join(
            dirs['abs_test_install_dir'], 'mochitest')
        dirs['abs_instrumentation_dir'] = os.path.join(
            dirs['abs_test_install_dir'], 'instrumentation')
        dirs['abs_blob_upload_dir'] = os.path.join(abs_dirs['abs_work_dir'], 'blobber_upload_dir')
        dirs['abs_jittest_dir'] = os.path.join(dirs['abs_test_install_dir'], "jit-test", "jit-test")
        dirs['shutdown_dir'] = abs_dirs['abs_work_dir'].rsplit("/", 2)[0]
        dirs['abs_cppunittest_dir'] = os.path.join(
            dirs['abs_test_install_dir'], 'cppunittest')
        for key in dirs.keys():
            if key not in abs_dirs:
                abs_dirs[key] = dirs[key]
        self.abs_dirs = abs_dirs
        return self.abs_dirs

    def _query_symbols_url(self):
        """query the full symbols URL based upon binary URL"""
        # may break with name convention changes but is one less 'input' for script
        if self.symbols_url:
            return self.symbols_url

    def _query_abs_base_cmd(self, suite_category, suite):
        #check for apk first with if ?
        c = self.config
        dirs = self.query_abs_dirs()
        options = []
        run_file = c["suite_definitions"][suite_category]["run_filename"]
        base_cmd = ['python', '-u']
        base_cmd.append(os.path.join((dirs["abs_%s_dir" % suite_category]), run_file))
        self.device_ip = socket.gethostbyname(self.mozpool_device)
        #applies to mochitest, reftest, jsreftest
        # TestingMixin._download_and_extract_symbols() will set
        # self.symbols_path when downloading/extracting.
        hostnumber = 0
        mozpool_device_list = self.mozpool_device.split('-')
        if len(mozpool_device_list) == 2:
            hostnumber = int(mozpool_device_list[1])
        http_port = '30%03i' % hostnumber
        ssl_port = '31%03i' % hostnumber
        #get filename from installer_url
        self.filename_apk = self.installer_url.split('/')[-1]
        #find appname from package-name.txt - assumes download-and-extract has completed successfully
        apk_dir = self.abs_dirs['abs_work_dir']
        self.apk_path = os.path.join(apk_dir, self.filename_apk)
        unzip = self.query_exe("unzip")
        package_path = os.path.join(apk_dir, 'package-name.txt')
        unzip_cmd = [unzip, '-q', '-o',  self.apk_path]
        self.run_command(unzip_cmd, cwd=apk_dir, halt_on_failure=True, fatal_exit_code=3)
        self.app_name = str(self.read_from_file(package_path, verbose=True)).rstrip()

        raw_log_file = os.path.join(dirs['abs_blob_upload_dir'],
                                    '%s_raw.log' % suite)
        error_summary_file = os.path.join(dirs['abs_blob_upload_dir'],
                                          '%s_errorsummary.log' % suite)
        str_format_values = {
            'device_ip': self.device_ip,
            'hostname': self.mozpool_device,
            'symbols_path': self._query_symbols_url(),
            'http_port': http_port,
            'ssl_port':  ssl_port,
            'app_name':  self.app_name,
            'apk_name':  self.filename_apk,
            'apk_path':  self.apk_path,
            'raw_log_file': raw_log_file,
            'error_summary_file': error_summary_file,
        }
        if "suite_definitions" in c and \
                suite_category in c["suite_definitions"]: # new in-tree format
            for option in c["suite_definitions"][suite_category]["options"]:
                options.append(option % str_format_values)
            abs_base_cmd = base_cmd + options
            return abs_base_cmd
        else:
            self.warning("Suite options for %s could not be determined."
                         "\nIf you meant to have options for this suite, "
                         "please make sure they are specified in your "
                         "config." % suite_category)

    ###### helper methods
    def _pre_config_lock(self, rw_config):
        super(PandaTest, self)._pre_config_lock(rw_config)
        c = self.config
        if not c.get('run_all_suites'):
            return  # configs are valid
        for category in SUITE_CATEGORIES:
            specific_suites = c.get('specified_%s_suites' % (category))
            if specific_suites:
                if specific_suites != 'all':
                    self.fatal("Config options are not valid. Please ensure"
                               " that if the '--run-all-suites' flag was enabled,"
                               " then do not specify to run only specific suites "
                               "like:\n '--mochitest-suite browser-chrome'")

    def close_request(self):
        if self.request_url:
            mph = self.query_mozpool_handler(self.mozpool_device)
            mph.close_request(self.request_url)
            self.info("Request '%s' deleted on cleanup" % self.request_url)
            self.request_url = None
        else:
            self.info("request_url doesn't exist. Already closed?")

    def _build_arg(self, option, value):
        """
        Build a command line argument
        """
        if not value:
            return []
        return [str(option), str(value)]

if __name__ == '__main__':
    pandaTest = PandaTest()
    pandaTest.run_and_exit()



#!/usr/bin/env python
import os
config = {
    "default_actions": [
        'clobber',
        'checkout-sources',
        'get-blobs',
        'update-source-manifest',
        'build',
        'build-symbols',
        'make-updates',
        'prep-upload',
        'upload',
        # bug 1222227 - temporarily disable for S3 migration
        # 'make-socorro-json',
        # 'upload-source-manifest',
        'submit-to-balrog',
    ],
    "upload": {
        "default": {
            "ssh_key": os.path.expanduser("~/.ssh/b2gbld_dsa"),
            "ssh_user": "b2gbld",
            "upload_remote_host": "pvtbuilds.pvt.build.mozilla.org",
            "upload_remote_path": "/pvt/mozilla.org/b2gotoro/tinderbox-builds/%(branch)s-%(target)s/%(buildid)s",
            "upload_remote_symlink": "/pvt/mozilla.org/b2gotoro/tinderbox-builds/%(branch)s-%(target)s/latest",
            "upload_remote_nightly_path": "/pvt/mozilla.org/b2gotoro/nightly/%(branch)s-%(target)s/%(year)04i/%(month)02i/%(year)04i-%(month)02i-%(day)02i-%(hour)02i-%(minute)02i-%(second)02i",
            "upload_remote_nightly_symlink": "/pvt/mozilla.org/b2gotoro/nightly/%(branch)s-%(target)s/latest",
            "upload_dep_target_exclusions": ["inari", "leo", "helix", "mako", "wasabi"],
        },
        "public": {
            "ssh_key": os.path.expanduser("~/.ssh/ffxbld_rsa"),
            "ssh_user": "ffxbld",
            "upload_remote_host": "upload.ffxbld.productdelivery.prod.mozaws.net",
            "post_upload_cmd": "post_upload.py --tinderbox-builds-dir %(branch)s-%(target)s -p b2g -i %(buildid)s --revision %(revision)s --release-to-tinderbox-dated-builds",
            "post_upload_nightly_cmd": "post_upload.py --tinderbox-builds-dir %(branch)s-%(target)s -b %(branch)s-%(target)s -p b2g -i %(buildid)s --revision %(revision)s --release-to-tinderbox-dated-builds --release-to-latest --release-to-dated",
        },
    },
    "gittool_share_base": "/builds/git-shared/git",
    "gittool_base_mirror_urls": [],
    "hgtool_share_base": "/builds/hg-shared",
    "hgtool_base_bundle_urls": ["https://ftp-ssl.mozilla.org/pub/mozilla.org/firefox/bundles"],
    "exes": {
        "tooltool.py": "/tools/tooltool.py",
    },
    "manifest": {
        "upload_remote_host": "stage.mozilla.org",
        "upload_remote_basepath": "/pub/mozilla.org/b2g/manifests/nightly/%(version)s",
        "depend_upload_remote_basepath": "/pub/mozilla.org/b2g/manifests/depend/%(branch)s/%(platform)s/%(buildid)s",
        "ssh_key": os.path.expanduser("~/.ssh/b2gbld_dsa"),
        "ssh_user": "b2gbld",
        "branches": {
            'mozilla-b2g37_v2_2': '2.2.0',
            'mozilla-b2g44_v2_5': '2.5.0',
            'mozilla-central': '2.6.0',
        },
        "translate_hg_to_git": True,
        "translate_base_url": "http://cruncher.build.mozilla.org/mapper/{project}/{vcs}/{rev}",
    },
    "env": {
        "CCACHE_DIR": "/builds/ccache",
        "CCACHE_COMPRESS": "1",
        "CCACHE_UMASK": "002",
        "GAIA_OPTIMIZE": "1",
        "SYMBOL_SERVER_HOST": "symbolpush.mozilla.org",
        "SYMBOL_SERVER_USER": "b2gbld",
        "SYMBOL_SERVER_SSH_KEY": "/home/mock_mozilla/.ssh/b2gbld_dsa",
        "SYMBOL_SERVER_PATH": "/mnt/netapp/breakpad/symbols_b2g/",
        "POST_SYMBOL_UPLOAD_CMD": "/usr/local/bin/post-symbol-upload.py",
        "WGET_OPTS": "-c -q",
        "PATH": "/tools/python27/bin:%(PATH)s",
    },
    "purge_minsize": 15,
    "clobberer_url": "https://api.pub.build.mozilla.org/clobberer/lastclobber",
    "is_automation": True,
    "repo_mirror_dir": "/builds/git-shared/repo",
    "repo_remote_mappings": {
        'https://android.googlesource.com/': 'https://git.mozilla.org/external/aosp',
        'git://codeaurora.org/': 'https://git.mozilla.org/external/caf',
        'https://git.mozilla.org/b2g': 'https://git.mozilla.org/b2g',
        'git://github.com/mozilla-b2g/': 'https://git.mozilla.org/b2g',
        'git://github.com/mozilla/': 'https://git.mozilla.org/b2g',
        'https://git.mozilla.org/releases': 'https://git.mozilla.org/releases',
        'http://android.git.linaro.org/git-ro/': 'https://git.mozilla.org/external/linaro',
        'git://github.com/apitrace/': 'https://git.mozilla.org/external/apitrace',
    },
}



#!/usr/bin/env python
# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****
"""Code to tie into buildbot.
Ideally this will go away if and when we retire buildbot.
"""

import os
import re
import sys

try:
    import simplejson as json
    assert json
except ImportError:
    import json

sys.path.insert(1, os.path.dirname(sys.path[0]))

from mozharness.base.config import parse_config_file
from mozharness.base.log import INFO, WARNING, ERROR

# BuildbotMixin {{{1

TBPL_SUCCESS = 'SUCCESS'
TBPL_WARNING = 'WARNING'
TBPL_FAILURE = 'FAILURE'
TBPL_EXCEPTION = 'EXCEPTION'
TBPL_RETRY = 'RETRY'
TBPL_STATUS_DICT = {
    TBPL_SUCCESS: INFO,
    TBPL_WARNING: WARNING,
    TBPL_FAILURE: ERROR,
    TBPL_EXCEPTION: ERROR,
    TBPL_RETRY: WARNING,
}
EXIT_STATUS_DICT = {
    TBPL_SUCCESS: 0,
    TBPL_WARNING: 1,
    TBPL_FAILURE: 2,
    TBPL_EXCEPTION: 3,
    TBPL_RETRY: 4,
}
TBPL_WORST_LEVEL_TUPLE = (TBPL_RETRY, TBPL_EXCEPTION, TBPL_FAILURE,
                          TBPL_WARNING, TBPL_SUCCESS)


class BuildbotMixin(object):
    buildbot_config = None
    buildbot_properties = {}
    worst_buildbot_status = TBPL_SUCCESS

    def read_buildbot_config(self):
        c = self.config
        if not c.get("buildbot_json_path"):
            # If we need to fail out, add postflight_read_buildbot_config()
            self.info("buildbot_json_path is not set.  Skipping...")
        else:
            # TODO try/except?
            self.buildbot_config = parse_config_file(c['buildbot_json_path'])
            self.info("Using buildbot properties:")
            self.info(json.dumps(self.buildbot_config, indent=4))

    def tryserver_email(self):
        pass

    def buildbot_status(self, tbpl_status, level=None, set_return_code=True):
        if tbpl_status not in TBPL_STATUS_DICT:
            self.error("buildbot_status() doesn't grok the status %s!" % tbpl_status)
        else:
            # Set failure if our log > buildbot_max_log_size (bug 876159)
            if self.config.get("buildbot_max_log_size") and self.log_obj:
                # Find the path to the default log
                dirs = self.query_abs_dirs()
                log_file = os.path.join(
                    dirs['abs_log_dir'],
                    self.log_obj.log_files[self.log_obj.log_level]
                )
                if os.path.exists(log_file):
                    file_size = os.path.getsize(log_file)
                    if file_size > self.config['buildbot_max_log_size']:
                        self.error("Log file size %d is greater than max allowed %d! Setting TBPL_FAILURE (was %s)..." % (file_size, self.config['buildbot_max_log_size'], tbpl_status))
                        tbpl_status = TBPL_FAILURE
            if not level:
                level = TBPL_STATUS_DICT[tbpl_status]
            self.worst_buildbot_status = self.worst_level(tbpl_status, self.worst_buildbot_status, TBPL_WORST_LEVEL_TUPLE)
            if self.worst_buildbot_status != tbpl_status:
                self.info("Current worst status %s is worse; keeping it." % self.worst_buildbot_status)
            self.add_summary("# TBPL %s #" % self.worst_buildbot_status, level=level)
            if set_return_code:
                self.return_code = EXIT_STATUS_DICT[self.worst_buildbot_status]

    def set_buildbot_property(self, prop_name, prop_value, write_to_file=False):
        self.info("Setting buildbot property %s to %s" % (prop_name, prop_value))
        self.buildbot_properties[prop_name] = prop_value
        if write_to_file:
            return self.dump_buildbot_properties(prop_list=[prop_name], file_name=prop_name)
        return self.buildbot_properties[prop_name]

    def query_buildbot_property(self, prop_name):
        return self.buildbot_properties.get(prop_name)

    def query_is_nightly(self):
        """returns whether or not the script should run as a nightly build.

           First will check for 'nightly_build' in self.config and if that is
           not True, we will also allow buildbot_config to determine
           for us. Failing all of that, we default to False.
           Note, dependancy on buildbot_config is being deprecated.
           Putting everything in self.config is the preference.
        """
        if self.config.get('nightly_build'):
            return True
        elif self.buildbot_config and 'properties' in self.buildbot_config:
            return self.buildbot_config['properties'].get('nightly_build', False)
        else:
            return False

    def dump_buildbot_properties(self, prop_list=None, file_name="properties", error_level=ERROR):
        c = self.config
        if not os.path.isabs(file_name):
            file_name = os.path.join(c['base_work_dir'], "properties", file_name)
        dir_name = os.path.dirname(file_name)
        if not os.path.isdir(dir_name):
            self.mkdir_p(dir_name)
        if not prop_list:
            prop_list = self.buildbot_properties.keys()
            self.info("Writing buildbot properties to %s" % file_name)
        else:
            if not isinstance(prop_list, (list, tuple)):
                self.log("dump_buildbot_properties: Can't dump non-list prop_list %s!" % str(prop_list), level=error_level)
                return
            self.info("Writing buildbot properties %s to %s" % (str(prop_list), file_name))
        contents = ""
        for prop in prop_list:
            contents += "%s:%s\n" % (prop, self.buildbot_properties.get(prop, "None"))
        return self.write_to_file(file_name, contents)

    def invoke_sendchange(self, downloadables=None, branch=None,
                   username="sendchange-unittest", sendchange_props=None):
        """ Generic sendchange, currently b2g- and unittest-specific.
            """
        c = self.config
        buildbot = self.query_exe("buildbot", return_type="list")
        if branch is None:
            if c.get("debug_build"):
                platform = re.sub('[_-]debug', '', self.buildbot_config["properties"]["platform"])
                branch = '%s-%s-debug-unittest' % (self.buildbot_config["properties"]["branch"], platform)
            else:
                branch = '%s-%s-opt-unittest' % (self.buildbot_config["properties"]["branch"], self.buildbot_config["properties"]["platform"])
        sendchange = [
            'sendchange',
            '--master', c.get("sendchange_masters")[0],
            '--username', username,
            '--branch', branch,
        ]
        if self.buildbot_config['sourcestamp'].get("revision"):
            sendchange += ['-r', self.buildbot_config['sourcestamp']["revision"]]
        if len(self.buildbot_config['sourcestamp']['changes']) > 0:
            if self.buildbot_config['sourcestamp']['changes'][0].get('who'):
                sendchange += ['--username', self.buildbot_config['sourcestamp']['changes'][0]['who']]
            if self.buildbot_config['sourcestamp']['changes'][0].get('comments'):
                sendchange += ['--comments', self.buildbot_config['sourcestamp']['changes'][0]['comments'].encode('ascii', 'ignore')]
        if sendchange_props:
            for key, value in sendchange_props.iteritems():
                sendchange.extend(['--property', '%s:%s' % (key, value)])
        else:
            if self.buildbot_config["properties"].get("builduid"):
                sendchange += ['--property', "builduid:%s" % self.buildbot_config["properties"]["builduid"]]
            sendchange += [
                '--property', "buildid:%s" % self.query_buildid(),
                '--property', 'pgo_build:False',
            ]

        for d in downloadables:
            sendchange += [d]

        retcode = self.run_command(buildbot + sendchange)
        if retcode != 0:
            self.info("The sendchange failed but we don't want to turn the build orange: %s" % retcode)

    def query_build_name(self):
        build_name = self.config.get('platform')
        if not build_name:
            self.fatal('Must specify "platform" in the mozharness config for indexing')

        return build_name

    def query_build_type(self):
        if self.config.get('build_type'):
            build_type = self.config['build_type']
        elif self.config.get('pgo_build'):
            build_type = 'pgo'
        elif self.config.get('debug_build', False):
            build_type = 'debug'
        else:
            build_type = 'opt'
        return build_type

    def buildid_to_dict(self, buildid):
        """Returns an dict with the year, month, day, hour, minute, and second
           as keys, as parsed from the buildid"""
        buildidDict = {}
        try:
            # strptime is no good here because it strips leading zeros
            buildidDict['year'] = buildid[0:4]
            buildidDict['month'] = buildid[4:6]
            buildidDict['day'] = buildid[6:8]
            buildidDict['hour'] = buildid[8:10]
            buildidDict['minute'] = buildid[10:12]
            buildidDict['second'] = buildid[12:14]
        except:
            self.fatal('Could not parse buildid into YYYYMMDDHHMMSS: %s' % buildid)
        return buildidDict



config = {
  'build_command': "build.b2g",
  'expect_file': "expect.b2g.json",

  'default_actions': [
      'clobber',
      'checkout-tools',
      'checkout-sources',
      'get-blobs',
      'clobber-shell',
      'configure-shell',
      'build-shell',
      'clobber-analysis',
      'setup-analysis',
      'run-analysis',
      'collect-analysis-output',
      'upload-analysis',
      'check-expectations',
   ],

    'sixgill_manifest': "build/sixgill-b2g.manifest",
    'b2g_target_compiler_prefix': "target_compiler/gcc/linux-x86/arm/arm-linux-androideabi-4.7/bin/arm-linux-androideabi-",
}



import os

config = {
    'default_actions': [
        'clobber',
        'clone-tools',
        'checkout-sources',
        'build',
        'upload-files',
        'sendchange',
    ],
    'stage_platform': 'win32-mulet',
    'publish_nightly_en_US_routes': False,
    'build_type': 'mulet-opt',
    'stage_product': 'b2g',
    'tooltool_manifest_src': "b2g/dev/config/tooltool-manifests/win32/releng.manifest",
    'enable_signing': False,
    'enable_talos_sendchange': False,
    'enable_unittest_sendchange': False,
    #### 32 bit build specific #####
    'env': {
        'BINSCOPE': 'C:/Program Files (x86)/Microsoft/SDL BinScope/BinScope.exe',
        'HG_SHARE_BASE_DIR': 'C:/builds/hg-shared',
        'MOZBUILD_STATE_PATH': os.path.join(os.getcwd(), '.mozbuild'),
        'MOZ_AUTOMATION': '1',
        'MOZ_CRASHREPORTER_NO_REPORT': '1',
        'MOZ_OBJDIR': 'obj-firefox',
        'PATH': 'C:/mozilla-build/nsis-3.0b1;C:/mozilla-build/python27;'
                'C:/mozilla-build/buildbotve/scripts;'
                '%s' % (os.environ.get('path')),
        'PROPERTIES_FILE': os.path.join(os.getcwd(), 'buildprops.json'),
        'TINDERBOX_OUTPUT': '1',
        'XPCOM_DEBUG_BREAK': 'stack-and-abort',
    },
    'src_mozconfig': 'b2g/dev/config/mozconfigs/win32/mulet',
    'purge_minsize': 9,
    #######################
}



#!/usr/bin/env python
import os
config = {
    "default_actions": [
        'clobber',
        'checkout-sources',
        'get-blobs',
        'update-source-manifest',
        'build',
        'build-symbols',
        'prep-upload',
        'upload',
    ],
    "upload": {
        "default": {
            "ssh_key": os.path.expanduser("~/.ssh/b2gtry_dsa"),
            "ssh_user": "b2gtry",
            "upload_remote_host": "pvtbuilds2.dmz.scl3.mozilla.com",
            "upload_remote_path": "/pub/mozilla.org/b2g/try-builds/%(user)s-%(revision)s/%(branch)s-%(target)s",
            "upload_dep_target_exclusions": [],
        },
    },
    "gittool_share_base": "/builds/git-shared/git",
    "gittool_base_mirror_urls": [],
    "hgtool_share_base": "/builds/hg-shared",
    "hgtool_base_bundle_urls": ["https://ftp-ssl.mozilla.org/pub/mozilla.org/firefox/bundles"],
    "sendchange_masters": ["buildbot-master81.build.mozilla.org:9301"],
    "exes": {
        "tooltool.py": "/tools/tooltool.py",
        "buildbot": "/tools/buildbot/bin/buildbot",
    },
    "env": {
        "CCACHE_DIR": "/builds/ccache",
        "CCACHE_COMPRESS": "1",
        "CCACHE_UMASK": "002",
        "GAIA_OPTIMIZE": "1",
        "WGET_OPTS": "-c -q",
        "PATH": "/tools/python27/bin:%(PATH)s",
    },
    "purge_minsize": 20,
    #"clobberer_url": "https://api-pub-build.allizom.org/clobberer/lastclobber",
    #"clobberer_url": "https://api.pub.build.mozilla.org/clobberer/lastclobber",
    "is_automation": True,
    "force_clobber": True,
    "repo_mirror_dir": "/builds/git-shared/repo",
    "repo_remote_mappings": {
        'https://android.googlesource.com/': 'https://git.mozilla.org/external/aosp',
        'git://codeaurora.org/': 'https://git.mozilla.org/external/caf',
        'https://git.mozilla.org/b2g': 'https://git.mozilla.org/b2g',
        'git://github.com/mozilla-b2g/': 'https://git.mozilla.org/b2g',
        'git://github.com/mozilla/': 'https://git.mozilla.org/b2g',
        'https://git.mozilla.org/releases': 'https://git.mozilla.org/releases',
        'http://android.git.linaro.org/git-ro/': 'https://git.mozilla.org/external/linaro',
        'git://github.com/apitrace/': 'https://git.mozilla.org/external/apitrace',
    },
}



config = {
    'base_name': 'Android armv7 API 11+ b2gdroid %(branch)s',
    'stage_platform': 'android-api-11-b2gdroid',
    'build_type': 'api-11-b2gdroid-opt',
    'src_mozconfig': 'mobile/android/b2gdroid/config/mozconfigs/nightly',
    'tooltool_manifest_src': 'mobile/android/config/tooltool-manifests/b2gdroid/releng.manifest',
    'multi_locale_config_platform': 'android',
    'enable_nightly_promotion': True,
}



#!/usr/bin/env python
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import copy
import os
import sys
from datetime import datetime
from functools import wraps

sys.path.insert(1, os.path.dirname(sys.path[0]))

from mozharness.base.errors import MakefileErrorList
from mozharness.mozilla.blob_upload import BlobUploadMixin, blobupload_config_options
from mozharness.mozilla.building.buildb2gbase import B2GBuildBaseScript, B2GMakefileErrorList
from mozharness.mozilla.purge import PurgeMixin
from mozharness.mozilla.building.hazards import HazardAnalysis, HazardError

SUCCESS, WARNINGS, FAILURE, EXCEPTION, RETRY = xrange(5)

nuisance_env_vars = ['TERMCAP', 'LS_COLORS', 'PWD', '_']


def requires(*queries):
    """Wrapper for detecting problems where some bit of information
    required by the wrapped step is unavailable. Use it put prepending
    @requires("foo"), which will check whether self.query_foo() returns
    something useful."""
    def make_wrapper(f):
        @wraps(f)
        def wrapper(self, *args, **kwargs):
            for query in queries:
                val = query(self)
                assert (val is not None and "None" not in str(val)), "invalid " + query.__name__
            return f(self, *args, **kwargs)
        return wrapper
    return make_wrapper


class B2GHazardBuild(PurgeMixin, B2GBuildBaseScript, BlobUploadMixin):
    all_actions = [
        'clobber',
        'checkout-tools',
        'checkout-sources',
        'get-blobs',
        'update-source-manifest',
        'clobber-shell',
        'configure-shell',
        'build-shell',
        'clobber-analysis',
        'setup-analysis',
        'run-analysis',
        'collect-analysis-output',
        'upload-analysis',
        'check-expectations',
    ]

    default_actions = [
        'clobber',
        'checkout-tools',
        'checkout-sources',
        'get-blobs',
        'clobber-shell',
        'configure-shell',
        'build-shell',
        'clobber-analysis',
        'setup-analysis',
        'run-analysis',
        'collect-analysis-output',
        'upload-analysis',
        'check-expectations',
    ]

    def __init__(self):
        super(B2GHazardBuild, self).__init__(
            config_options=copy.deepcopy(blobupload_config_options),
            config={
                'default_vcs': 'hgtool',
                'ccache': False,
                'mozilla_dir': 'build/gecko',

                'upload_ssh_server': None,
                'upload_remote_basepath': None,
                'enable_try_uploads': True,
            },
            all_actions=B2GHazardBuild.all_actions,
            default_actions=B2GHazardBuild.default_actions,
        )

        self.buildid = None
        self.create_virtualenv()
        self.analysis = HazardAnalysis()

    def _pre_config_lock(self, rw_config):
        super(B2GHazardBuild, self)._pre_config_lock(rw_config)

        if self.buildbot_config:
            self.config['is_automation'] = True
            self.config.setdefault('blob_upload_branch', self.config.get('branch', self.buildbot_config['properties']['branch']))
        else:
            self.config['is_automation'] = False
            self.config.setdefault('blob_upload_branch', 'devel')

        dirs = self.query_abs_dirs()
        replacements = self.config['env_replacements'].copy()
        for k, v in replacements.items():
            replacements[k] = v % dirs

        self.env = self.query_env(replace_dict=replacements,
                                  partial_env=self.config['partial_env'],
                                  purge_env=nuisance_env_vars)
        self.env['MOZ_UPLOAD_DIR'] = dirs['abs_blob_upload_dir']

    def query_abs_dirs(self):
        if self.abs_dirs:
            return self.abs_dirs
        abs_dirs = super(B2GHazardBuild, self).query_abs_dirs()

        abs_work_dir = abs_dirs['abs_work_dir']
        dirs = {
            'b2g_src':
                abs_work_dir,
            'target_compiler_base':
                os.path.join(abs_dirs['abs_work_dir'], 'target_compiler'),
            'shell_objdir':
                os.path.join(abs_work_dir, self.config['shell-objdir']),
            'mozharness_scriptdir':
                os.path.abspath(os.path.dirname(__file__)),
            'abs_analysis_dir':
                os.path.join(abs_work_dir, self.config['analysis-dir']),
            'abs_analyzed_objdir':
                os.path.join(abs_work_dir, self.config['srcdir'], self.config['analysis-objdir']),
            'analysis_scriptdir':
                os.path.join(abs_dirs['gecko_src'], self.config['analysis-scriptdir']),
            'abs_blob_upload_dir':
                os.path.join(abs_work_dir, 'blobber_upload_dir'),
        }

        abs_dirs.update(dirs)
        self.abs_dirs = abs_dirs
        return self.abs_dirs

    def query_branch(self):
        if self.buildbot_config and 'properties' in self.buildbot_config:
            return self.buildbot_config['properties']['branch']
        elif 'branch' in self.config:
            # Used for locally testing try vs non-try
            return self.config['branch']
        else:
            return os.path.basename(self.query_repo())

    def query_compiler_manifest(self):
        dirs = self.query_abs_dirs()
        return os.path.join(dirs['analysis_scriptdir'], self.config['compiler_manifest'])

    def query_b2g_compiler_manifest(self):
        dirs = self.query_abs_dirs()
        return os.path.join(dirs['analysis_scriptdir'], self.config['b2g_compiler_manifest'])

    def query_sixgill_manifest(self):
        dirs = self.query_abs_dirs()
        return os.path.join(dirs['analysis_scriptdir'], self.config['sixgill_manifest'])

    def query_buildid(self):
        if self.buildid:
            return self.buildid
        if self.buildbot_config and 'properties' in self.buildbot_config:
            self.buildid = self.buildbot_config['properties'].get('buildid')
        if not self.buildid:
            self.buildid = datetime.now().strftime("%Y%m%d%H%M%S")
        return self.buildid

    def query_upload_ssh_server(self):
        if self.buildbot_config and 'properties' in self.buildbot_config:
            return self.buildbot_config['properties']['upload_ssh_server']
        else:
            return self.config['upload_ssh_server']

    def query_upload_ssh_key(self):
        if self.buildbot_config and 'properties' in self.buildbot_config:
            key = self.buildbot_config['properties']['upload_ssh_key']
        else:
            key = self.config['upload_ssh_key']
        if self.mock_enabled and not key.startswith("/"):
            key = "/home/mock_mozilla/.ssh/" + key
        return key

    def query_upload_ssh_user(self):
        if self.buildbot_config and 'properties' in self.buildbot_config:
            return self.buildbot_config['properties']['upload_ssh_user']
        else:
            return self.config['upload_ssh_user']

    def query_product(self):
        if self.buildbot_config and 'properties' in self.buildbot_config:
            return self.buildbot_config['properties']['product']
        else:
            return self.config['product']

    def query_upload_remote_basepath(self):
        if self.config.get('upload_remote_basepath'):
            return self.config['upload_remote_basepath']
        else:
            return "/pub/mozilla.org/{product}".format(
                product=self.query_product(),
            )

    def query_upload_remote_baseuri(self):
        baseuri = self.config.get('upload_remote_baseuri')
        if self.buildbot_config and 'properties' in self.buildbot_config:
            buildprops = self.buildbot_config['properties']
            if 'upload_remote_baseuri' in buildprops:
                baseuri = buildprops['upload_remote_baseuri']
        return baseuri.strip("/") if baseuri else None

    def query_upload_label(self):
        if self.buildbot_config and 'properties' in self.buildbot_config:
            return self.buildbot_config['properties']['platform']
        else:
            return self.config.get('upload_label')

    def query_upload_path(self):
        branch = self.query_branch()

        common = {
            'basepath': self.query_upload_remote_basepath(),
            'branch': branch,
            'target': self.query_upload_label(),
        }

        if branch == 'try':
            if not self.config['enable_try_uploads']:
                return None
            try:
                user = self.buildbot_config['sourcestamp']['changes'][0]['who']
            except (KeyError, TypeError):
                user = "unknown"
            return "{basepath}/try-builds/{user}-{rev}/{branch}-{target}".format(
                user=user,
                rev=self.query_revision(),
                **common
            )
        else:
            return "{basepath}/tinderbox-builds/{branch}-{target}/{buildid}".format(
                buildid=self.query_buildid(),
                **common
            )

    def query_do_upload(self):
        if self.query_branch() == 'try':
            return self.config.get('enable_try_uploads')
        return True

    def make_source_dir(self):
        dirs = self.query_abs_dirs()
        dest = dirs['b2g_src']
        if not os.path.exists(dest):
            self.mkdir_p(dest)

    # The gecko config is required before enabling mock, because it determines
    # what mock target to use.
    def enable_mock(self):
        self.load_gecko_config()
        super(B2GHazardBuild, self).enable_mock()

    # Actions {{{2
    def clobber(self):
        dirs = self.query_abs_dirs()
        PurgeMixin.clobber(
            self,
            always_clobber_dirs=[
                dirs['abs_upload_dir'],
            ],
        )

    def checkout_sources(self):
        self.make_source_dir()
        super(B2GHazardBuild, self).checkout_sources()

    def get_blobs(self):
        dirs = self.query_abs_dirs()
        self.tooltool_fetch(self.query_compiler_manifest(),
                            output_dir=dirs['abs_work_dir'])
        self.tooltool_fetch(self.query_sixgill_manifest(),
                            output_dir=dirs['abs_work_dir'])
        if not os.path.exists(dirs['target_compiler_base']):
            self.mkdir_p(dirs['target_compiler_base'])
        self.tooltool_fetch(self.query_b2g_compiler_manifest(),
                            output_dir=dirs['target_compiler_base'])

    def clobber_shell(self):
        dirs = self.query_abs_dirs()
        self.rmtree(dirs['shell_objdir'])

    def configure_shell(self):
        self.enable_mock()
        dirs = self.query_abs_dirs()

        if not os.path.exists(dirs['shell_objdir']):
            self.mkdir_p(dirs['shell_objdir'])

        js_src_dir = os.path.join(dirs['gecko_src'], 'js', 'src')
        rc = self.run_command(['autoconf-2.13'],
                              cwd=js_src_dir,
                              env=self.env,
                              error_list=MakefileErrorList)
        if rc != 0:
            self.fatal("autoconf failed, can't continue.", exit_code=FAILURE)

        rc = self.run_command([os.path.join(js_src_dir, 'configure'),
                               '--enable-optimize',
                               '--disable-debug',
                               '--enable-ctypes',
                               '--with-system-nspr',
                               '--without-intl-api'],
                              cwd=dirs['shell_objdir'],
                              env=self.env,
                              error_list=MakefileErrorList)
        if rc != 0:
            self.fatal("Configure failed, can't continue.", exit_code=FAILURE)

        self.disable_mock()

    def build_shell(self):
        self.enable_mock()
        dirs = self.query_abs_dirs()

        rc = self.run_command(['make', '-j', str(self.config.get('concurrency', 4)), '-s'],
                              cwd=dirs['shell_objdir'],
                              env=self.env,
                              error_list=MakefileErrorList)
        if rc != 0:
            self.fatal("Build failed, can't continue.", exit_code=FAILURE)

        self.disable_mock()

    def clobber_analysis(self):
        dirs = self.query_abs_dirs()
        self.rmtree(dirs['abs_analysis_dir'])
        self.rmtree(dirs['abs_analyzed_objdir'])

    def setup_analysis(self):
        dirs = self.query_abs_dirs()
        analysis_dir = dirs['abs_analysis_dir']

        if not os.path.exists(analysis_dir):
            self.mkdir_p(analysis_dir)

        values = {
            'js': os.path.join(dirs['shell_objdir'], 'dist', 'bin', 'js'),
            'analysis_scriptdir': dirs['analysis_scriptdir'],
            'source_objdir': dirs['abs_analyzed_objdir'],
            'source': os.path.join(dirs['work_dir'], 'source'),
            'sixgill': os.path.join(dirs['work_dir'], self.config['sixgill']),
            'sixgill_bin': os.path.join(dirs['work_dir'], self.config['sixgill_bin']),
        }
        defaults = """
js = '%(js)s'
analysis_scriptdir = '%(analysis_scriptdir)s'
objdir = '%(source_objdir)s'
source = '%(source)s'
sixgill = '%(sixgill)s'
sixgill_bin = '%(sixgill_bin)s'
jobs = 2
""" % values

        #defaults_path = os.path.join(analysis_dir, 'defaults.py')
        defaults_path = os.path.join(analysis_dir, 'defaults.py')
        file(defaults_path, "w").write(defaults)
        self.log("Wrote analysis config file " + defaults_path)

        build_command = self.config['build_command']
        self.copyfile(os.path.join(dirs['mozharness_scriptdir'],
                                   os.path.join('spidermonkey', build_command)),
                      os.path.join(analysis_dir, build_command),
                      copystat=True)

    def run_analysis(self):
        dirs = self.query_abs_dirs()

        env = self.query_build_env().copy()
        self.enable_mock()

        gonk_misc = os.path.join(dirs['b2g_src'], 'gonk-misc')
        mozconfig = os.path.join(gonk_misc, 'hazard-analysis-config')
        mozconfig_text = ''
        mozconfig_text += 'CXXFLAGS="-Wno-attributes"\n'
        mozconfig_text += '. "%s/default-gecko-config"\n' % gonk_misc
        basecc = os.path.join(dirs['abs_work_dir'], self.config['sixgill'], 'scripts', 'wrap_gcc', 'basecc')
        mozconfig_text += "ac_add_options --with-compiler-wrapper=" + basecc + "\n"
        mozconfig_text += "ac_add_options --without-ccache\n"
        file(mozconfig, "w").write(mozconfig_text)

        # Stuff I set in my .userconfig for manual builds
        env['B2G_SOURCE'] = dirs['b2g_src']
        env['MOZCONFIG_PATH'] = mozconfig
        env['GECKO_PATH'] = dirs['gecko_src']
        env['TARGET_TOOLS_PREFIX'] = os.path.join(dirs['abs_work_dir'], self.config['b2g_target_compiler_prefix'])

        try:
            self.analysis.run(self, env=env, error_list=B2GMakefileErrorList)
        except HazardError as e:
            self.fatal(e, exit_code=FAILURE)

        self.disable_mock()

    def collect_analysis_output(self):
        self.analysis.collect_output(self)

    def upload_analysis(self):
        if not self.query_do_upload():
            self.info("Uploads disabled for this build. Skipping...")
            return

        self.enable_mock()

        try:
            self.analysis.upload_results(self)
        except HazardError as e:
            self.error(e)
            self.return_code = WARNINGS

        self.disable_mock()

    def check_expectations(self):
        try:
            self.analysis.check_expectations(self)
        except HazardError as e:
            self.fatal(e, exit_code=FAILURE)

# main {{{1
if __name__ == '__main__':
    myScript = B2GHazardBuild()
    myScript.run_and_exit()



# This is a template config file for b2g emulator unittest testing

config = {
    # mozharness script options
    "application": "b2g",
    "test_suite": "xpcshell",                               # reftest, mochitest or xpcshell

    "emulator_url": "http://127.0.1.1/b2g/emulator.zip",    # url to emulator zip file
    "installer_url": "http://127.0.1.1/b2g/b2g.tar.gz",     # url to gecko build
    "xre_url": "http://127.0.1.1/b2g/xpcshell.zip",         # url to xpcshell zip file
    "test_url": "http://127.0.1.1/b2g/tests.zip",           # url to tests.zip
    "busybox_url": "http://127.0.1.1/b2g/busybox",          # url to busybox binary

    # testsuite options
    #"adb_path": "path/to/adb",           # defaults - os.environ['ADB_PATH']
    #"test_manifest": "path/to/manifest", # defaults - mochitest: "b2g.json"
                                          #              reftest: "tests/layout/reftests/reftest.list"
                                          #             xpcshell: "tests/xpcshell.ini"
    "total_chunks": 1,
    "this_chunk": 1,
}



#!/usr/bin/env python
# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****
""" b2g_branch_repos.py

Branch the b2g git repos for merge day, and lock down the 3rd party repos
in b2g-manifest to a revision.
"""

import glob
import os
import pprint
import sys
import time

sys.path.insert(1, os.path.dirname(os.path.dirname(sys.path[0])))

from mozharness.base.errors import GitErrorList
from mozharness.base.log import ERROR
from mozharness.base.transfer import TransferMixin
from mozharness.base.vcs.vcsbase import VCSScript
from mozharness.mozilla import repo_manifest


# B2GBranch {{{1
class B2GBranch(TransferMixin, VCSScript):
    config_options = [
        [['--branch-name', ], {
            "action": "store",
            "dest": "branch_name",
            "type": "string",
            "help": "Specify the branch name (e.g. v2.0).  REQUIRED",
        }],
        [['--branch-order', ], {
            "action": "extend",
            "dest": "branch_order",
            "type": "string",
            "help": """Comma separated list of branches in case manifests contain conflicting references,
e.g. --branch-order v2.0,master"""
        }],
        [['--manifest-repo-revision', ], {
            "action": "store",
            "dest": "manifest_repo_revision",
            "type": "string",
            "default": "master",
            "help": "Specify the b2g-manifest repo revision"
        }],
        [['--lock-manifest-revisions', ], {
            "action": "store_true",
            "dest": "lock_manifest_revisions",
            "default": True,
            "help": "Lock 3rd party revisions in munge-manifests"
        }],
        [['--no-lock-manifest-revisions', ], {
            "action": "store_false",
            "dest": "lock_manifest_revisions",
            "help": "Don't lock 3rd party revisions in munge-manifests"
        }],
        [['--branch-manifests', ], {
            "action": "store_true",
            "dest": "branch_manifests",
            "default": True,
            "help": "branch b2g-manifest and xml files in munge-manifests"
        }],
        [['--no-branch-manifests', ], {
            "action": "store_false",
            "dest": "branch_manifests",
            "help": "Don't branch b2g-manifest and xml files in munge-manifests"
        }],
        [['--delete-unused-manifests', ], {
            "action": "store_true",
            "dest": "delete_unused_manifests",
            "default": True,
            "help": "Delete unused manifest files in munge-manifests"
        }],
        [['--no-delete-unused-manifests', ], {
            "action": "store_false",
            "dest": "delete_unused_manifests",
            "help": "Don't delete unused manifest files in munge-manifests"
        }],
    ]
    branch_repo_dict = None
    remote_branch_revisions = {}

    def __init__(self, require_config_file=True):
        super(B2GBranch, self).__init__(
            config_options=self.config_options,
            all_actions=[
                'clobber',
                'pull',
                'pull-branch-repos',
                'clean-repos',
                'branch-repos',
                'switch-manifest-branch',
                'munge-manifests',
                'push',
            ],
            default_actions=[
                'pull',
                'pull-branch-repos',
                'clean-repos',
                'branch-repos',
                'switch-manifest-branch',
                'munge-manifests',
            ],
            require_config_file=require_config_file
        )
        self.run_sanity_check()

# Helper methods {{{1
    def run_sanity_check(self):
        message = ""
        if not self.config.get("branch_name"):
            message += "You need to specify --branch-name (e.g. v2.0)!\n"
        if message:
            self.fatal(message)

    def query_abs_dirs(self):
        if self.abs_dirs:
            return self.abs_dirs
        super(B2GBranch, self).query_abs_dirs()
        self.abs_dirs['abs_manifest_dir'] = os.path.join(self.abs_dirs['abs_work_dir'], 'b2g-manifest')
        return self.abs_dirs

    def query_manifests(self):
        dirs = self.query_abs_dirs()
        manifests = []
        for manifest in glob.glob(os.path.join(dirs['abs_manifest_dir'], "*.xml")):
            if os.path.islink(manifest):
                self.info("Skipping %s (softlink)" % manifest)
                continue
            if os.path.basename(manifest) in self.config["ignored_manifests"]:
                self.info("Skipping %s (ignored)" % manifest)
                continue
            manifests.append(manifest)
        return manifests

    def _add_branch_revision(self, repo_config, name, revision, manifest):
        """ query_branch_repos() helper.

            Pass in branch-repos['name'] for 'repo', name, revision, manifest.
            """
        if repo_manifest.is_commitid(revision):
            self.debug("%s is a commit id! Skipping for %s %s" % (revision, name, manifest))
            return
        new_branch = self.config['branch_name']
        branch_order = [new_branch] + list(self.config.get('branch_order', []))
        repo_config.setdefault('all_revisions', {}).setdefault(revision, []).append(manifest)
        if "revision" not in repo_config:
            repo_config['revision'] = revision
        else:
            previous_revision = repo_config["revision"]
            if previous_revision != revision:
                if revision in branch_order:
                    if previous_revision not in branch_order or \
                       branch_order.index(revision) < branch_order.index(previous_revision):
                        self.info("Prefering %s over %s for %s" % (revision,
                                  previous_revision, name))
                        del(repo_config["branch_revisions"][previous_revision])
                        repo_config["revision"] = revision
                    else:
                        self.info("%s: not branching off %s since %s is in branch_order" % (name, revision, previous_revision))
                        return
                elif previous_revision in branch_order:
                    self.info("%s: not keeping %s since %s is in branch_order" % (name, revision, previous_revision))
                    self.debug(pprint.pformat(repo_config['branch_revisions']))
                    return
                else:
                    # We shouldn't ever hit this line
                    self.critical("WAT %s %s %s" % (name, revision, previous_revision))
        repo_config.setdefault('branch_revisions', {}).setdefault(revision, []).append(manifest)
        self.debug("%s %s %s" % (name, repo_config['revision'], pprint.pformat(repo_config['branch_revisions'])))

    def _query_remote_branch_revision(self, fetch, branch, manifest, retry=True):
        """ Helper method for munge_manifests().

            Since we want to lock revisions, we need to know what revision goes
            with a specific git url + branch.
            """
        sleep_time = 10
        if self.remote_branch_revisions.get(fetch, {}).get(branch):
            return self.remote_branch_revisions[fetch][branch]
        git = self.query_exe('git', return_type='list')
        output = self.get_output_from_command(
            git + ["ls-remote", fetch, branch]
        )
        if not output:
            if retry:
                self.info("Trying again in %d seconds..." % sleep_time)
                time.sleep(sleep_time)
                return self._query_remote_branch_revision(manifest, fetch, branch, retry=False)
            # TODO figure out what to do here
            # self.fatal("Can't find revision for %s %s!" % (fetch, branch))
            self.critical("%s: Can't find revision for %s %s!" % (fetch, branch, manifest))
            return None
        r = output[0:40]
        if not repo_manifest.is_commitid(r):
            self.fatal("Can't grok ls-remote output:\n %s" % output)
        self.info("Found revision %s for %s %s" % (r, fetch, branch))
        self.remote_branch_revisions.setdefault(fetch, {})[branch] = r
        return r

    def _query_repo_name(self, name_attr):
        """ query_branch_repos() helper, so gaia and gaia.git are equivalent
            """
        if name_attr.endswith(".git"):
            name_attr = name_attr[:-4]
        return name_attr

    def _query_do_branch(self, fetch, name):
        """ query_branch_repos() helper, to see if we want to [potentially]
            branch a repo based on the 'fetch' url
            """
        do_branch = False
        for rstring in self.config.get("branch_remote_substrings", []):
            if rstring in fetch:
                do_branch = True
                break
        if do_branch:
            if name in self.config.get('no_branch_repos', []):
                self.info("Skipping %s (in no_branch_repos)" % name)
                return False
            return True
        self.debug("Not going to branch %s (not in branch_remote_substrings)" % name)
        return False

    def query_branch_repos(self):
        """ Parse all manifests and build a dictionary of repos with
            expected revisions and/or branches.

            The format will be {
                name: {
                    'revision': branch,
                    'fetch': git_url,
                    'branch_revisions': {
                        revision: [manifest list]  # This should only have one key/value pair
                    },
                    'all_revisions': {
                        revision: [manifest list]  # This will have all key/value pairs
                    },
                },
            }

            This depends on the pull action having run at least once.
            """
        if self.branch_repo_dict is not None:
            return self.branch_repo_dict
        self.info("Building branch_repo_dict...")
        branch_repo_dict = {
            'b2g-manifest': {
                'fetch': self.config['manifest_repo_url'],
                'revision': self.config['manifest_repo_revision'],
            },
        }
        for manifest in self.query_manifests():
            self.info("Processing %s" % manifest)
            doc = repo_manifest.load_manifest(manifest)
            try:
                default_revision = repo_manifest.get_default(doc).getAttribute("revision")
            except IndexError:
                self.info("Skipping %s (no defaults)" % manifest)
                continue

            for p in doc.getElementsByTagName('project'):
                name = self._query_repo_name(p.getAttribute('name'))
                fetch = repo_manifest.get_project_remote_url(doc, p)
                self.debug("Remote %s Name %s" % (fetch, name))
                # We branch github.com/mozilla repos only
                if not self._query_do_branch(fetch, name):
                    continue
                # Now start building the branch info
                branch_repo_dict.setdefault(name, {}).setdefault("fetch", fetch)
                revision = p.getAttribute("revision")
                if not revision:
                    # fall back to default revision
                    if default_revision:
                        self.info("%s: %s, using default revision (%s)" % (manifest, name, default_revision))
                        self._add_branch_revision(branch_repo_dict[name], name, default_revision, manifest)
                    else:
                        self.warning("Can't determine revision for %s in %s" % (name, manifest))
                elif revision and branch_repo_dict.get(name, {}).get("revision", revision) != revision:
                    self._add_branch_revision(branch_repo_dict[name], name, revision, manifest)
                else:
                    self._add_branch_revision(branch_repo_dict[name], name, revision, manifest)
        self.info("Outputting branch_repo_dict:")
        self.info(pprint.pformat(branch_repo_dict))
        message = ""
        for name, r in branch_repo_dict.iteritems():
            if r.get("revision") is None:
                self.warning("Sanity: No revision set for %s %s; we'll fall back to master" % (name, r["fetch"]))
            if len(r.get("branch_revisions", {})) > 1:
                message += "Sanity: Not clear where to branch for %s %s %s\n" % (name, r["fetch"], pprint.pformat(r["branch_revisions"]))
        if message:
            self.fatal(message + "Use --branch-order or self.config['no_branch_repos'] to fix!")
        self.branch_repo_dict = branch_repo_dict
        return branch_repo_dict

    def _get_sha_from_show_ref(self, output):
        """ Helper method for check_existing_branch().

            'output' is the output from |git show-ref|.
            """
        if not output or 'refs' not in output:
            return None
        return output[0:41]

    def check_existing_branch(self, new_branch, cwd, error_level=ERROR):
        """ Returns a tuple (local, remote) with the corresponding sha
            of each branch if they exist, or None if not.
            """
        self.info("Checking for existing branch %s in %s..." % (new_branch, cwd))
        git = self.query_exe('git', return_type='list')
        local_branch = self._get_sha_from_show_ref(
            self.get_output_from_command(
                git + ["show-ref", new_branch],
                cwd=cwd,
                success_codes=[0, 1],
            )
        )
        remote_branch = self._get_sha_from_show_ref(
            self.get_output_from_command(
                git + ["show-ref", "remotes/origin/%s" % new_branch],
                cwd=cwd,
                success_codes=[0, 1],
            )
        )
        if local_branch and remote_branch and local_branch != remote_branch:
            self.log(
                "%s out of sync! Local branch %s on %s; remote on %s!" % (
                    cwd, new_branch, local_branch, remote_branch
                ),
                level=error_level,
            )
        return (local_branch, remote_branch)

    def _delete_unused_manifests(self, unused_manifests):
        """ Helper method for munge_manifests().  Delete unused manifests
            if self.config["delete_unused_manifests"]; otherwise skip.
            """
        self.info("Deleting unused manifests...")
        git = self.query_exe("git", return_type="list")
        for manifest in unused_manifests:
            cwd = os.path.dirname(manifest)
            self.run_command(
                git + ['rm', '-f', os.path.basename(manifest)],
                cwd=cwd,
                error_list=GitErrorList,
                halt_on_failure=True,
            )

# Actions {{{1
    def pull(self):
        """ Pull action.
            """
        dirs = self.query_abs_dirs()
        super(B2GBranch, self).pull(
            repos=[{
                "vcs": "hg",
                "repo": self.config["tools_repo_url"],
                "revision": self.config["tools_repo_revision"],
                "dest": "tools",
            }, {
                "vcs": "gittool",
                "repo": self.config["manifest_repo_url"],
                "revision": self.config["manifest_repo_revision"],
                "dest": dirs['abs_manifest_dir'],
            }],
        )

    def pull_branch_repos(self):
        repos = []
        for name, conf in self.query_branch_repos().iteritems():
            self.info("%s: %s" % (name, pprint.pformat(conf)))
            repos.append({
                "vcs": "gittool",
                "repo": conf['fetch'],
                "revision": conf.get('revision', 'master'),
                "dest": name,
            })
        super(B2GBranch, self).pull(repos=repos)

    def clean_repos(self):
        """ Clean the existing branches.
            This assumes the branches were created locally and not pushed remote yet.
            """
        repos = self.query_branch_repos()
        dirs = self.query_abs_dirs()
        git = self.query_exe("git", return_type="list")
        new_branch = self.config['branch_name']
        # b2g-manifest is special
        if os.path.exists(dirs['abs_manifest_dir']):
            for cmd in (
                git + ["reset", "--hard"],
                git + ["clean", "-fdx"],
                git + ["checkout", self.config["manifest_repo_revision"]],
            ):
                self.run_command(
                    cmd,
                    cwd=dirs["abs_manifest_dir"],
                    error_list=GitErrorList,
                    halt_on_failure=True,
                )
        for name, r in repos.iteritems():
            cwd = os.path.join(dirs['abs_work_dir'], name)
            local_sha, remote_sha = self.check_existing_branch(new_branch, cwd)
            if local_sha and not remote_sha:
                # We have a local branch that doesn't exist on origin
                self.info("Cleaning up existing %s branch from %s." % (new_branch, name))
                self.run_command(
                    git + ['branch', '-D', new_branch],
                    cwd=os.path.join(dirs['abs_work_dir'], name),
                    halt_on_failure=True,
                    error_list=GitErrorList,
                )

    def branch_repos(self):
        """ Branch the mozilla-b2g repos and update the manifests.
            """
        repos = self.query_branch_repos()
        dirs = self.query_abs_dirs()
        git = self.query_exe("git", return_type="list")
        new_branch = self.config['branch_name']
        for name, r in repos.iteritems():
            revision = r["revision"]
            cwd = os.path.join(dirs['abs_work_dir'], name)
            local_sha, remote_sha = self.check_existing_branch(new_branch, cwd)
            if local_sha:
                self.warning("Local branch %s already exists in %s. Skipping." % (new_branch, name))
                continue
            if remote_sha:
                self.error("Remote branch %s already exists in %s. Do you need to re- --pull-branch-repos? Skipping." % (new_branch, name))
                continue
            self.info("Creating %s branch in %s based on %s" % (new_branch, name,
                      revision))
            self.run_command(git + ['branch', new_branch, "origin/%s" % revision],
                             cwd=cwd, error_list=GitErrorList, halt_on_failure=True)

    def switch_manifest_branch(self):
        """ Switch b2g-manifest to the new branch.
            This is in a separate action because we may not run --branch-repos
            (e.g. if we're locking down revisions in a previously created branch)
            and switching branches after making changes can be detrimental to
            our previous changes.

            This action will die if there are uncommitted changes, currently
            by design.
            """
        dirs = self.query_abs_dirs()
        git = self.query_exe("git", return_type="list")
        new_branch = self.config['branch_name']
        self.run_command(
            git + ["checkout", new_branch],
            cwd=dirs['abs_manifest_dir'],
            error_list=GitErrorList,
            halt_on_failure=True
        )

    def munge_manifests(self):
        """ Switch the branched repos to the new branch; lock down third
            party revisions.
            """
        branch_repos = self.query_branch_repos()
        dirs = self.query_abs_dirs()
        new_branch = self.config['branch_name']
        unused_manifests = []
        if not self.check_existing_branch(new_branch, cwd=dirs['abs_manifest_dir'])[0]:
            self.fatal("b2g-manifest isn't branched properly!  Run --clean-repos --branch-repos")
        for manifest in self.query_manifests():
            self.info("Munging %s..." % manifest)
            doc = repo_manifest.load_manifest(manifest)
            try:
                repo_manifest.get_default(doc).getAttribute("revision")
            except IndexError:
                self.info("No default revision; skipping.")
                unused_manifests.append(manifest)
                continue
            for p in doc.getElementsByTagName('project'):
                name = self._query_repo_name(p.getAttribute('name'))
                fetch = repo_manifest.get_project_remote_url(doc, p)
                self.debug("Remote %s Name %s" % (fetch, name))
                current_revision = repo_manifest.get_project_revision(doc, p)
                if repo_manifest.is_commitid(current_revision):
                    self.info("%s: %s is already locked to %s; skipping." % (manifest, name, current_revision))
                    # I could setAttribute() here, but I very much doubt the
                    # default_revision is a commitid.
                    continue
                # We've branched this repo; do we set the revision to
                # new_branch or not?  ('fetch' needs to match, since we have
                # same-named repos with different urls =P )
                if name in branch_repos and branch_repos[name]['fetch'] == fetch:
                    orig_branch = branch_repos[name]['branch_revisions'].keys()[0]
                    if manifest in branch_repos[name]['branch_revisions'][orig_branch]:
                        if current_revision != orig_branch:
                            self.fatal("I don't know how we got here, but %s in %s's revision %s is not the branching point %s." %
                                       (name, manifest, current_revision, orig_branch))
                        self.info("Setting %s (%s) to %s (was %s)" % (name, manifest, new_branch, current_revision))
                        p.setAttribute('revision', new_branch)
                        continue
                    # Should we keep the old branch or lock revision?  Doing
                    # the former for now.
                    self.info("%s %s is off a different branch point (%s, not %s).  Keeping the old branch..." %
                              (manifest, name, current_revision, orig_branch))
                    continue
                if name in self.config['extra_branch_manifest_repos']:
                    p.setAttribute('revision', new_branch)
                    continue
                # Lock revision?
                if not self.config["lock_manifest_revisions"]:
                    self.info("%s: Not locking revision for %s due to config." % (manifest, name))
                    continue
                lock_revision = self._query_remote_branch_revision(fetch, current_revision, manifest)
                if lock_revision is not None:
                    p.setAttribute('revision', lock_revision)
            with self.opened(manifest, open_mode='w') as (fh, err):
                if err:
                    self.fatal("Can't open %s for writing!" % manifest)
                else:
                    doc.writexml(fh)
            fh.close()
        if self.config["delete_unused_manifests"]:
            self._delete_unused_manifests(unused_manifests)
        self.info("TODO: diff, commit, --push!")

    def push(self):
        """ Push the repos.
            """
        repos = self.query_branch_repos()
        dirs = self.query_abs_dirs()
        git = self.query_exe("git", return_type="list")
        new_branch = self.config['branch_name']
        problem_repos = []
        for name, r in repos.iteritems():
            fetch = r["fetch"]
            fetch = fetch.replace("git://github.com/", "git@github.com:")
            cwd = os.path.join(dirs['abs_work_dir'], name)
            self.info("pushing %s" % name)
            status = self.run_command(
                git + ['push', fetch, "%s:%s" % (new_branch, new_branch)],
                cwd=cwd, error_list=GitErrorList
            )
            if status:
                problem_repos.append(fetch)
        if problem_repos:
            self.error("Had problems with the following repos:\n%s" % pprint.pformat(problem_repos))


# __main__ {{{1
if __name__ == '__main__':
    b2g_branch = B2GBranch()
    b2g_branch.run_and_exit()



#!/usr/bin/env python
# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****

import sys
import os

# load modules from parent dir
sys.path.insert(1, os.path.dirname(sys.path[0]))

# import the guts
from mozharness.base.config import parse_config_file
from mozharness.base.errors import MakefileErrorList
from mozharness.base.log import ERROR, FATAL
from mozharness.base.script import BaseScript
from mozharness.base.vcs.vcsbase import VCSMixin
from mozharness.mozilla.l10n.locales import GaiaLocalesMixin, LocalesMixin


class B2gMultilocale(LocalesMixin, BaseScript, VCSMixin, GaiaLocalesMixin):
    """ This is a helper script that requires MercurialBuildFactory
        logic to work.  We may eventually make this a standalone
        script.

        We could inherit MercurialScript instead of BaseScript + VCSMixin
    """
    config_options = [
        [["--locale"], {
            "action": "extend",
            "dest": "locales",
            "type": "string",
            "help": "Specify the locale(s) to repack"
        }],
        [["--gaia-languages-file"], {
            "dest": "gaia_languages_file",
            "help": "languages file for gaia multilocale profile",
        }],
        [["--gecko-languages-file"], {
            "dest": "locales_file",
            "help": "languages file for gecko multilocale",
        }],
        [["--gecko-l10n-root"], {
            "dest": "hg_l10n_base",
            "help": "root location for gecko l10n repos",
        }],
        [["--gecko-l10n-base-dir"], {
            "dest": "l10n_dir",
            "help": "dir to clone gecko l10n repos into, relative to the work directory",
        }],
        [["--merge-locales"], {
            "dest": "merge_locales",
            "help": "Dummy option to keep from burning. We now always merge",
        }],
        [["--gaia-l10n-root"], {
            "dest": "gaia_l10n_root",
            "help": "root location for gaia l10n repos",
        }],
        [["--gaia-l10n-base-dir"], {
            "dest": "gaia_l10n_base_dir",
            "default": "build-gaia-l10n",
            "help": "dir to clone l10n repos into, relative to the work directory",
        }],
        [["--gaia-l10n-vcs"], {
            "dest": "gaia_l10n_vcs",
            "help": "vcs to use for gaia l10n",
        }],
    ]

    def __init__(self, require_config_file=False):
        LocalesMixin.__init__(self)
        BaseScript.__init__(self,
                            config_options=self.config_options,
                            all_actions=[
                                'pull',
                                'build',
                                'summary',
                            ],
                            require_config_file=require_config_file,

                            # Default configuration
                            config={
                                'gaia_l10n_vcs': 'hg',
                                'vcs_share_base': os.environ.get('HG_SHARE_BASE_DIR'),
                                'locales_dir': 'b2g/locales',
                                'l10n_dir': 'gecko-l10n',
                                # I forget what this was for.  Copied from the Android multilocale stuff
                                'ignore_locales': ["en-US", "multi"],
                                # This only has 2 locales in it.  We probably need files that mirror gaia's locale lists
                                # We need 2 sets of locales files because the locale names in gaia are different than gecko, e.g. 'es' vs 'es-ES'
                                # We'll need to override this for localizer buidls
                                'locales_file': 'build/b2g/locales/all-locales',
                                'mozilla_dir': 'build',
                                'objdir': 'obj-firefox',
                                'merge_locales': True,
                                'work_dir': '.',
                                'vcs_output_timeout': 600,  # 10 minutes should be enough for anyone!
                            },
                            )

    def _pre_config_lock(self, rw_config):
        super(B2gMultilocale, self)._pre_config_lock(rw_config)

        if 'pull' in self.actions:
            message = ""
            if 'gaia_languages_file' not in self.config:
                message += 'Must specify --gaia-languages-file!\n'
            if 'gaia_l10n_root' not in self.config:
                message += 'Must specify --gaia-l10n-root!\n'
            if 'locales_file' not in self.config:
                message += 'Must specify --gecko-languages-file!\n'
            if 'hg_l10n_base' not in self.config:
                message += 'Must specify --gecko-l10n-root!\n'
            if message:
                self.fatal(message)

    def query_abs_dirs(self):
        if self.abs_dirs:
            return self.abs_dirs
        abs_dirs = LocalesMixin.query_abs_dirs(self)

        c = self.config
        dirs = {
            'src': os.path.join(c['work_dir'], 'gecko'),
            'work_dir': abs_dirs['abs_work_dir'],
            'gaia_l10n_base_dir': os.path.join(abs_dirs['abs_work_dir'], self.config['gaia_l10n_base_dir']),
            'abs_compare_locales_dir': os.path.join(abs_dirs['base_work_dir'], 'compare-locales'),
        }

        abs_dirs.update(dirs)
        self.abs_dirs = abs_dirs
        return self.abs_dirs

    # Actions {{{2
    def pull(self):
        """ Clone gaia and gecko locale repos
        """
        languages_file = self.config['gaia_languages_file']
        l10n_base_dir = self.query_abs_dirs()['gaia_l10n_base_dir']
        l10n_config = {
            'root': self.config['gaia_l10n_root'],
            'vcs': self.config['gaia_l10n_vcs'],
        }

        self.pull_gaia_locale_source(l10n_config, parse_config_file(languages_file).keys(), l10n_base_dir)
        self.pull_locale_source()
        gecko_locales = self.query_locales()
        # populate b2g/overrides, which isn't in gecko atm
        dirs = self.query_abs_dirs()
        for locale in gecko_locales:
            self.mkdir_p(os.path.join(dirs['abs_l10n_dir'], locale, 'b2g', 'chrome', 'overrides'))
            self.copytree(os.path.join(dirs['abs_l10n_dir'], locale, 'mobile', 'overrides'),
                          os.path.join(dirs['abs_l10n_dir'], locale, 'b2g', 'chrome', 'overrides'),
                          error_level=FATAL)

    def build(self):
        """ Do the multilocale portion of the build + packaging.
        """
        dirs = self.query_abs_dirs()
        gecko_locales = self.query_locales()
        make = self.query_exe('make', return_type='string')
        env = self.query_env(
            partial_env={
                'LOCALE_MERGEDIR': dirs['abs_merge_dir'],
                'MOZ_CHROME_MULTILOCALE': 'en-US ' + ' '.join(gecko_locales),
            }
        )
        merge_env = self.query_env(
            partial_env={
                'PATH': '%(PATH)s' + os.pathsep + os.path.join(dirs['abs_compare_locales_dir'], 'scripts'),
                'PYTHONPATH': os.path.join(dirs['abs_compare_locales_dir'],
                                           'lib'),

            }
        )
        for locale in gecko_locales:
            command = make + ' merge-%s L10NBASEDIR=%s LOCALE_MERGEDIR=%s' % (locale, dirs['abs_l10n_dir'], dirs['abs_merge_dir'])
            status = self.run_command(command,
                                      cwd=dirs['abs_locales_dir'],
                                      error_list=MakefileErrorList,
                                      env=merge_env)
            command = make + ' chrome-%s L10NBASEDIR=%s LOCALE_MERGEDIR=%s' % (locale, dirs['abs_l10n_dir'], dirs['abs_merge_dir'])
            status = self.run_command(command,
                                      cwd=dirs['abs_locales_dir'],
                                      error_list=MakefileErrorList)
            if status:
                self.add_summary("Failed to add locale %s!" % locale, level=ERROR)
        command = make + ' -C app'
        self.run_command(command,
                         cwd=os.path.join(dirs['abs_objdir'], 'b2g'),
                         error_list=MakefileErrorList,
                         halt_on_failure=True)
        command = make + ' package AB_CD=multi'
        self.run_command(command, cwd=dirs['abs_objdir'],
                         error_list=MakefileErrorList, env=env,
                         halt_on_failure=True)
        command = make + ' package-tests AB_CD=multi'
        self.run_command(command, cwd=dirs['abs_objdir'],
                         error_list=MakefileErrorList, env=env,
                         halt_on_failure=True)


# main {{{1
if __name__ == '__main__':
    myScript = B2gMultilocale()
    myScript.run_and_exit()



# This is a template config file for b2g emulator unittest testing

config = {
    # mozharness script options
    "installer_url": "https://ftp-ssl.mozilla.org/pub/mozilla.org/b2g/tinderbox-builds/mozilla-central-linux64_gecko/1365627906/b2g-23.0a1.multi.linux-x86_64.tar.bz2",
    "xre_url": "https://ftp-ssl.mozilla.org/pub/mozilla.org/xulrunner/nightly/2012/09/2012-09-20-03-05-43-mozilla-central/xulrunner-18.0a1.en-US.linux-x86_64.tar.bz2"
}



#!/usr/bin/env python
# Written for Mozilla by Chris AtLee <catlee@mozilla.com> 2008
"""Delete old buildbot builds to make room for the current build.

%prog [options] base_dir1 [base_dir2 ...]

base_dir1 is the root of the directory tree you want to delete builds
from.

Sub-directories of base_dir1 will be deleted, in order from oldest to newest,
until the specified amount of space is free.

base_dir1 will always be used for space calculations, but if other base_dir#
are provided, subdirectories within those dirs will also be purged. This will
obviously only increase the available space if the other base_dirs are on the
same mountpoint, but this can be useful for, e.g., cleaning up scratchbox.

example:
    python %prog -s 6 /builds/moz2_slave /scratchbox/users/cltbld/home/cltbld/build
"""

import os
import shutil
import sys
from fnmatch import fnmatch
import re

DEFAULT_BASE_DIRS = [".."]

clobber_suffix = '.deleteme'

if sys.platform == 'win32':
    # os.statvfs doesn't work on Windows
    from win32file import RemoveDirectory, DeleteFile, \
        GetFileAttributesW, SetFileAttributesW, GetDiskFreeSpace, \
        FILE_ATTRIBUTE_NORMAL, FILE_ATTRIBUTE_DIRECTORY
    from win32api import FindFiles

    def freespace(p):
        secsPerClus, bytesPerSec, nFreeClus, totClus = GetDiskFreeSpace(p)
        return secsPerClus * bytesPerSec * nFreeClus
else:
    def freespace(p):
        "Returns the number of bytes free under directory `p`"
        r = os.statvfs(p)
        return r.f_frsize * r.f_bavail


def mtime_sort(p1, p2):
    "sorting function for sorting a list of paths by mtime"
    return cmp(os.path.getmtime(p1), os.path.getmtime(p2))


def rmdirRecursiveWindows(dir):
    """Windows-specific version of rmdirRecursive that handles
    path lengths longer than MAX_PATH.
    """

    dir = os.path.realpath(dir)
    # Make sure directory is writable
    SetFileAttributesW('\\\\?\\' + dir, FILE_ATTRIBUTE_NORMAL)

    for ffrec in FindFiles('\\\\?\\' + dir + '\*.*'):
        file_attr = ffrec[0]
        name = ffrec[8]
        if name == '.' or name == '..':
            continue
        full_name = os.path.join(dir, name)

        if file_attr & FILE_ATTRIBUTE_DIRECTORY:
            rmdirRecursiveWindows(full_name)
        else:
            SetFileAttributesW('\\\\?\\' + full_name, FILE_ATTRIBUTE_NORMAL)
            DeleteFile('\\\\?\\' + full_name)
    RemoveDirectory('\\\\?\\' + dir)


def rmdirRecursive(dir):
    """This is a replacement for shutil.rmtree that works better under
    windows. Thanks to Bear at the OSAF for the code.
    (Borrowed from buildbot.slave.commands)"""
    if os.name == 'nt':
        rmdirRecursiveWindows(dir)
        return

    if not os.path.exists(dir):
        # This handles broken links
        if os.path.islink(dir):
            os.remove(dir)
        return

    if os.path.islink(dir):
        os.remove(dir)
        return

    # Verify the directory is read/write/execute for the current user
    os.chmod(dir, 0700)

    for name in os.listdir(dir):
        full_name = os.path.join(dir, name)
        # on Windows, if we don't have write permission we can't remove
        # the file/directory either, so turn that on
        if os.name == 'nt':
            if not os.access(full_name, os.W_OK):
                # I think this is now redundant, but I don't have an NT
                # machine to test on, so I'm going to leave it in place
                # -warner
                os.chmod(full_name, 0600)

        if os.path.isdir(full_name):
            rmdirRecursive(full_name)
        else:
            # Don't try to chmod links
            if not os.path.islink(full_name):
                os.chmod(full_name, 0700)
            os.remove(full_name)
    os.rmdir(dir)


def str2seconds(s):
    """ Accepts time intervals resembling:
         30d  (30 days)
         10h  (10 hours)
        Returns the specified interval as a positive integer in seconds.
    """
    m = re.match(r'^(\d+)([dh])$', s)
    if (m):
        mul = {'d': 24*60*60, 'h': 60*60}
        n = int(m.group(1))
        unit = m.group(2)
        return n * mul[unit]
    else:
        raise ValueError("Unhandled time format '%s'" % s)


def purge(base_dirs, gigs, ignore, max_age, dry_run=False):
    """Delete directories under `base_dirs` until `gigs` GB are free.

    Delete any directories older than max_age.

    Will not delete directories listed in the ignore list except
    those tagged with an expiry threshold.  Example:

      rel-*:40d

    Will not delete rel-* directories until they are over 40 days old.
    """
    gigs *= 1024 * 1024 * 1024

    # convert 'ignore' to a dict resembling { directory: cutoff_time }
    # where a cutoff time of -1 means 'never expire'.
    ignore = dict(map(lambda x: x.split(':')[0:2] if len(x.split(':')) > 1 else [x, -1], ignore))
    ignore = dict(map(lambda key: [key, time.time() - str2seconds(ignore[key])] if ignore[key] != -1 else [key, ignore[key]], ignore))

    dirs = []
    for base_dir in base_dirs:
        if os.path.exists(base_dir):
            for d in os.listdir(base_dir):
                p = os.path.join(base_dir, d)
                if not os.path.isdir(p):
                    continue
                mtime = os.path.getmtime(p)
                skip = False
                for pattern, cutoff_time in ignore.iteritems():
                    if (fnmatch(d, pattern)):
                        if cutoff_time == -1 or mtime > cutoff_time:
                            skip = True
                            break
                        else:
                            print("Ignored directory '%s' exceeds cutoff time" % d)
                if skip:
                    continue
                dirs.append((mtime, p))

    dirs.sort()

    while dirs:
        mtime, d = dirs.pop(0)

        # If we're newer than max_age, and don't need any more free space,
        # we're all done here
        if (not max_age) or (mtime > max_age):
            if freespace(base_dirs[0]) >= gigs:
                break

        print "Deleting", d
        if not dry_run:
            try:
                clobber_path = d + clobber_suffix
                if os.path.exists(clobber_path):
                    rmdirRecursive(clobber_path)
                # Prevent repeated moving.
                if d.endswith(clobber_suffix):
                    rmdirRecursive(d)
                else:
                    shutil.move(d, clobber_path)
                    rmdirRecursive(clobber_path)
            except:
                print >>sys.stderr, "Couldn't purge %s properly. Skipping." % d


def purge_hg_shares(share_dir, gigs, max_age, dry_run=False):
    """Deletes old hg directories under share_dir"""
    # Find hg directories
    hg_dirs = []
    for root, dirs, files in os.walk(share_dir):
        for d in dirs[:]:
            path = os.path.join(root, d, '.hg')
            if os.path.exists(path) or os.path.exists(path + clobber_suffix):
                hg_dirs.append(os.path.join(root, d))
                # Remove d from the list so we don't go traversing down into it
                dirs.remove(d)

    # Now we have a list of hg directories, call purge on them
    purge(hg_dirs, gigs, [], max_age, dry_run)

    # Clean up empty directories
    for d in hg_dirs:
        if not os.path.exists(os.path.join(d, '.hg')):
            print "Cleaning up", d
            if not dry_run:
                rmdirRecursive(d)

if __name__ == '__main__':
    import time
    from optparse import OptionParser
    from ConfigParser import ConfigParser, NoOptionError

    max_age = 14
    config = ConfigParser()
    config.read(os.path.expanduser('~/.purge_builds.cfg'))
    try:
        max_age = config.getint('DEFAULT', 'max_age')
    except (NoOptionError, ValueError):
        pass

    cwd = os.path.basename(os.getcwd())
    parser = OptionParser(usage=__doc__)
    parser.set_defaults(size=5, share_size=1, skip=[cwd], dry_run=False, max_age=max_age)

    parser.add_option('-s', '--size',
                      help='free space required (in GB, default 5)', dest='size',
                      type='float')

    parser.add_option('--share-size',
                      help='free space required for vcs shares (in GB, default 1)', dest='share_size',
                      type='float')

    parser.add_option('-n', '--not', help='do not delete this directory. Append :30d to skip for up to 30 days, or :30h to skip for up to 30 hours',
                      action='append', dest='skip')

    parser.add_option('', '--dry-run', action='store_true',
                      dest='dry_run',
                      help='''do not delete anything, just print out what would be
deleted.  note that since no directories are deleted, if the amount of free
disk space in base_dir(s) is less than the required size, then ALL directories
will be listed in the order in which they would be deleted.''')

    parser.add_option('', '--max-age', dest='max_age', type='int',
                      help='''maximum age (in days) for directories.  If any directory
            has an mtime older than this, it will be deleted, regardless of how
            much free space is required.  Set to 0 to disable.''')

    options, base_dirs = parser.parse_args()

    if len(base_dirs) < 1:
        for d in DEFAULT_BASE_DIRS:
            if os.path.exists(d):
                base_dirs.append(d)
    if len(base_dirs) < 1:
        parser.error("Must specify one or more base_dirs")
        sys.exit(1)

    # Figure out the mtime before which we'll start deleting old directories
    if options.max_age:
        cutoff_time = time.time() - 24 * 3600 * options.max_age
    else:
        cutoff_time = None

    purge(base_dirs, options.size, options.skip, cutoff_time, options.dry_run)

    # Try to cleanup shared hg repos. We run here even if we've freed enough
    # space so we can be sure and delete repositories older than max_age
    if 'HG_SHARE_BASE_DIR' in os.environ:
        purge_hg_shares(os.environ['HG_SHARE_BASE_DIR'],
                        options.share_size, cutoff_time, options.dry_run)

    after = freespace(base_dirs[0]) / (1024 * 1024 * 1024.0)

    # Try to cleanup the current dir if we still need space and it will
    # actually help.
    if after < options.size:
        # We skip the tools dir here because we've usually just cloned it.
        purge(['.'], options.size, ['tools'], cutoff_time, options.dry_run)
        after = freespace(base_dirs[0]) / (1024 * 1024 * 1024.0)

    if after < options.size:
        print "Error: unable to free %1.2f GB of space. " % options.size + \
              "Free space only %1.2f GB" % after
        sys.exit(1)
    else:
        print "%1.2f GB of space available" % after
        sys.exit(0)



import os
import sys

config = {
    "log_name": "b2g_branch_repos",

    "manifest_repo_url": "git://github.com/mozilla-b2g/b2g-manifest.git",
    "manifest_repo_revision": "master",
    "tools_repo_url": "https://hg.mozilla.org/build/tools",
    "tools_repo_revision": "default",
    "branch_remote_substrings": [
        'github.com/mozilla',
    ],
    "no_branch_repos": [
        "gecko",
        # device conflicts
        "platform_external_qemu",
        "codeaurora_kernel_msm",
        "platform_system_core",
        "platform_hardware_ril",
        "android-sdk",
        "platform_external_wpa_supplicant_8",
        "platform_prebuilts_misc",
        "hardware_qcom_display",
        "device_generic_goldfish",
        "platform_frameworks_av",
        "platform_frameworks_base",
        "android-development",
        "device-hammerhead",
        "device_lge_hammerhead-kernel",
        "platform_bionic",
        "platform_build",
    ],
    "extra_branch_manifest_repos": [
        "gecko",
        "gaia",
    ],
    "branch_order": [
        "master",
    ],
    # outdated/unsupported devices
    "ignored_manifests": [
        "dolphin.xml",
        "otoro.xml",
        "hamachi.xml",
        "helix.xml",
        "keon.xml",
        "peak.xml",
        "galaxy-nexus.xml",
        "galaxy-s2.xml",
        "nexus-s.xml",
        "pandaboard.xml",
        "wasabi.xml",
        "nexus-4.xml",
        "flo.xml",
        "flame.xml",
        "flame-l.xml",
        "nexus-s-4g.xml",
        "rpi.xml",
    ],
    "exes": {
        "hg": [
            "hg", "--config",
            "hostfingerprints.hg.mozilla.org=af:27:b9:34:47:4e:e5:98:01:f6:83:2b:51:c9:aa:d8:df:fb:1a:27",
        ],
        "gittool.py": [sys.executable, os.getcwd() + "/build/tools/buildfarm/utils/gittool.py"],
    },
}



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import mozversion
import logging
import os
import sys
import time
import traceback
import urllib
import utils
import mozhttpd

from talos.results import TalosResults
from talos.ttest import TTest
from talos.utils import TalosError, TalosCrash, TalosRegression
from talos.config import get_configs, ConfigurationError

# directory of this file
here = os.path.dirname(os.path.realpath(__file__))


def useBaseTestDefaults(base, tests):
    for test in tests:
        for item in base:
            if item not in test:
                test[item] = base[item]
                if test[item] is None:
                    test[item] = ''
    return tests


def buildCommandLine(test):
    """build firefox command line options for tp tests"""

    # sanity check pageloader values
    # mandatory options: tpmanifest, tpcycles
    if test['tpcycles'] not in range(1, 1000):
        raise TalosError('pageloader cycles must be int 1 to 1,000')
    if test.get('tpdelay') and test['tpdelay'] not in range(1, 10000):
        raise TalosError('pageloader delay must be int 1 to 10,000')
    if 'tpmanifest' not in test:
        raise TalosError("tpmanifest not found in test: %s" % test)

    # build pageloader command from options
    url = ['-tp', test['tpmanifest']]
    CLI_bool_options = ['tpchrome', 'tpmozafterpaint', 'tpdisable_e10s',
                        'tpnoisy', 'rss', 'tprender', 'tploadnocache',
                        'tpscrolltest']
    CLI_options = ['tpcycles', 'tppagecycles', 'tpdelay', 'tptimeout']
    for key in CLI_bool_options:
        if test.get(key):
            url.append('-%s' % key)
    for key in CLI_options:
        value = test.get(key)
        if value:
            url.extend(['-%s' % key, str(value)])

    # XXX we should actually return the list but since we abuse
    # the url as a command line flag to pass to firefox all over the place
    # will just make a string for now
    return ' '.join(url)


def setup_webserver(webserver):
    """use mozhttpd to setup a webserver"""
    logging.info("starting webserver on %r" % webserver)

    host, port = webserver.split(':')
    return mozhttpd.MozHttpd(host=host, port=int(port), docroot=here)


def run_tests(config, browser_config):
    """Runs the talos tests on the given configuration and generates a report.
    """
    # get the test data
    tests = config['tests']
    tests = useBaseTestDefaults(config.get('basetest', {}), tests)

    paths = ['profile_path', 'tpmanifest', 'extensions', 'setup', 'cleanup']
    for test in tests:

        # Check for profile_path, tpmanifest and interpolate based on Talos
        # root https://bugzilla.mozilla.org/show_bug.cgi?id=727711
        # Build command line from config
        for path in paths:
            if test.get(path):
                test[path] = utils.interpolate(test[path])
        if test.get('tpmanifest'):
            test['tpmanifest'] = \
                os.path.normpath('file:/%s' % (urllib.quote(test['tpmanifest'],
                                               '/\\t:\\')))
        if not test.get('url'):
            # build 'url' for tptest
            test['url'] = buildCommandLine(test)
        test['url'] = utils.interpolate(test['url'])
        test['setup'] = utils.interpolate(test['setup'])
        test['cleanup'] = utils.interpolate(test['cleanup'])

    # pass --no-remote to firefox launch, if --develop is specified
    # we do that to allow locally the user to have another running firefox
    # instance
    if browser_config['develop']:
        browser_config['extra_args'] = '--no-remote'

    # set defaults
    title = config.get('title', '')
    testdate = config.get('testdate', '')

    if browser_config['e10s'] and not title.endswith(".e"):
        # we are running in e10s mode
        title = "%s.e" % (title,)

    # get the process name from the path to the browser
    if not browser_config['process']:
        browser_config['process'] = \
            os.path.basename(browser_config['browser_path'])

    # fix paths to substitute
    # `os.path.dirname(os.path.abspath(__file__))` for ${talos}
    # https://bugzilla.mozilla.org/show_bug.cgi?id=705809
    browser_config['extensions'] = [utils.interpolate(i)
                                    for i in browser_config['extensions']]
    browser_config['bcontroller_config'] = \
        utils.interpolate(browser_config['bcontroller_config'])

    # normalize browser path to work across platforms
    browser_config['browser_path'] = \
        os.path.normpath(browser_config['browser_path'])

    binary = browser_config["browser_path"]
    version_info = mozversion.get_version(binary=binary)
    browser_config['browser_name'] = version_info['application_name']
    browser_config['browser_version'] = version_info['application_version']
    browser_config['buildid'] = version_info['application_buildid']
    try:
        browser_config['repository'] = version_info['application_repository']
        browser_config['sourcestamp'] = version_info['application_changeset']
    except KeyError:
        if not browser_config['develop']:
            print "unable to find changeset or repository: %s" % version_info
            sys.exit()
        else:
            browser_config['repository'] = 'develop'
            browser_config['sourcestamp'] = 'develop'

    # get test date in seconds since epoch
    if testdate:
        date = int(time.mktime(time.strptime(testdate,
                                             '%a, %d %b %Y %H:%M:%S GMT')))
    else:
        date = int(time.time())
    logging.debug("using testdate: %d", date)
    logging.debug("actual date: %d", int(time.time()))

    # results container
    talos_results = TalosResults(title=title,
                                 date=date,
                                 browser_config=browser_config)

    # results links
    if not browser_config['develop']:
        results_urls = dict(
            # hardcoded, this will be removed soon anyway.
            results_urls=['http://graphs.mozilla.org/server/collect.cgi'],
            # another hack; datazilla stands for Perfherder
            # and do not require url, but a non empty dict is required...
            datazilla_urls=['local.json'],
        )
    else:
        # local mode, output to files
        results_urls = dict(
            results_urls=[os.path.abspath('local.out')],
            datazilla_urls=[os.path.abspath('local.json')]
        )
    talos_results.check_output_formats(results_urls)

    httpd = setup_webserver(browser_config['webserver'])
    httpd.start()

    testname = None
    try:
        # run the tests
        timer = utils.Timer()
        logging.info("Starting test suite %s", title)
        for test in tests:
            testname = test['name']
            testtimer = utils.Timer()
            logging.info("Starting test %s", testname)

            mytest = TTest()
            talos_results.add(mytest.runTest(browser_config, test))

            logging.info("Completed test %s (%s)", testname,
                         testtimer.elapsed())

    except TalosRegression:
        logging.error("Detected a regression for %s", testname)
        # by returning 1, we report an orange to buildbot
        # http://docs.buildbot.net/latest/developer/results.html
        return 1
    except (TalosCrash, TalosError):
        # NOTE: if we get into this condition, talos has an internal
        # problem and cannot continue
        #       this will prevent future tests from running
        traceback.print_exception(*sys.exc_info())
        # indicate a failure to buildbot, turn the job red
        return 2
    finally:
        httpd.stop()

    logging.info("Completed test suite (%s)", timer.elapsed())

    # output results
    if results_urls:
        talos_results.output(results_urls)
        if browser_config['develop']:
            print
            print ("Thanks for running Talos locally. Results are in"
                   " %s and %s" % (results_urls['results_urls'],
                                   results_urls['datazilla_urls']))

    # we will stop running tests on a failed test, or we will return 0 for
    # green
    return 0


def main(args=sys.argv[1:]):
    try:
        config, browser_config = get_configs()
    except ConfigurationError, exc:
        sys.exit("ERROR: %s" % exc)
    utils.startLogger('debug' if config['debug'] else 'info')
    sys.exit(run_tests(config, browser_config))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****
"""b2g_tag.py

Tag the b2g repos for merge day.
"""

import os
import pprint
import sys
import time

try:
    import simplejson as json
    assert json
except ImportError:
    import json

sys.path.insert(1, os.path.dirname(os.path.dirname(sys.path[0])))

from mozharness.base.errors import GitErrorList, HgErrorList, VCSException
from mozharness.base.log import INFO
from mozharness.base.transfer import TransferMixin
from mozharness.base.vcs.vcsbase import MercurialScript


# B2GTag {{{1
class B2GTag(TransferMixin, MercurialScript):
    config_options = [
        [['--gecko-repo', ], {
            "action": "extend",
            "dest": "gecko_repos",
            "type": "string",
            "help": "Specify which gecko repo(s) to tag, along with gaia."
        }],
        [['--date-string', ], {
            "action": "store",
            "dest": "date_string",
            "type": "string",
            "default": time.strftime('%Y%m%d'),
            "help": "Specify the date string to use in the tag name."
        }],
    ]
    gecko_repos = None

    def __init__(self, require_config_file=True):
        super(B2GTag, self).__init__(
            config_options=self.config_options,
            all_actions=[
                'clobber',
                'clean-repos',
                'pull',
                'push-loop',
                'summary',
            ],
            default_actions=[
                'clean-repos',
                'pull',
                'push-loop',
                'summary',
            ],
            require_config_file=require_config_file
        )
        self.run_sanity_check()

# Helper methods {{{1
    def run_sanity_check(self):
        """ Make sure we're set up correctly before we start cloning or tagging
            anything.

            First, verify any specified gecko_repos exist in the b2g_branches
            dict.

            Next, make sure we can reach mapper and it's working.

            If there are any errors, compile the list and fatal().
            """
        sanity_message = ""
        if self.config.get("gecko_repos"):
            bad_branch_names = []
            for repo_name in self.query_gecko_repos():
                if repo_name not in self.config["b2g_branches"].keys():
                    bad_branch_names.append(repo_name)
            if bad_branch_names:
                sanity_message += "The following branch name(s) aren't in b2g_branches: %s\n" % ",".join(bad_branch_names)
        if 'push_loop' in self.actions:
            try:
                json_contents = self.load_json_from_url(
                    "%s/1" % self.config["gaia_mapper_base_url"],
                    log_level=INFO,
                )
                assert json_contents["git_rev"]
            except:
                sanity_message += "Can't load %s/1 for hg->git mapping!\n" % self.config["gaia_mapper_base_url"]
        if sanity_message:
            self.fatal(sanity_message)

    def query_gecko_repos(self):
        """ Which gecko repos + gaia are we converting?
            By default everything in self.config['b2g_branches'],
            but we can override that list with --gecko-repos.
            """
        if self.gecko_repos:
            return self.gecko_repos
        self.gecko_repos = list(self.config.get("gecko_repos",
                                self.config['b2g_branches'].keys()))
        self.gecko_repos.sort()
        return self.gecko_repos

    def query_repo_pull_config(self, repo_name, b2g_branch_config):
        """ Build the repo pull url for a repo. This is for convenience.
            """
        return {
            "repo": '%s/%s' % (self.config['hg_base_pull_url'], repo_name),
            "dest": repo_name,
            "tag": b2g_branch_config.get("tag", "default")
        }

    def query_repo_push_url(self, repo_name):
        """ Build the repo push url for a repo. This is for convenience.
            """
        return '%s/%s' % (self.config['hg_base_push_url'], repo_name)

    def query_gaia_git_revision(self, repo_name, b2g_branch_config):
        """ For most repos, read b2g/config/gaia.json,
            read the hg gaia revision, convert that in mapper, and return it.
            In the case of not being able to determine the git revision,
            throw a VCSException.

            Otherwise return None, and we'll tag the tip of the branch.
            """
        dirs = self.query_abs_dirs()
        if not b2g_branch_config.get("use_gaia_json", True):
            return None
        json_path = os.path.join(dirs["abs_work_dir"], repo_name, "b2g", "config", "gaia.json")
        if not os.path.exists(json_path):
            raise VCSException("%s doesn't exist!" % json_path)
        contents = self.read_from_file(json_path)
        try:
            json_contents = json.loads(contents)
            hg_revision = json_contents['revision']
        except ValueError:
            raise VCSException("%s is invalid json!" % json_path)
        except KeyError:
            raise VCSException("%s has no 'revision' set!" % json_path)
        try:
            url = "%s/%s" % (self.config["gaia_mapper_base_url"], hg_revision)
            json_contents = self.load_json_from_url(url)
            git_rev = json_contents["git_rev"]
            assert git_rev
            return git_rev
        except:
            raise VCSException("Unable to get git_rev from %s !" % url)

    def query_abs_dirs(self):
        """ Override the built-in query_abs_dirs to provide an absolute path
            for the gaia clone.
            """
        if self.abs_dirs:
            return self.abs_dirs
        dirs = super(B2GTag, self).query_abs_dirs()
        dirs['abs_gaia_dir'] = os.path.join(dirs['abs_work_dir'], 'gaia')
        self.abs_dirs = dirs
        return self.abs_dirs

    def query_tag_name(self, b2g_branch_config):
        """ Return the tag name for a specific branch config.
            e.g. B2G_1_3_20140428_MERGEDAY
            """
        return b2g_branch_config["tag_name"] % {'DATE': self.config['date_string']}

    def query_short_tag_name(self, b2g_branch_config):
        """ Return the shortened tag name for a specific branch config.
            This assumes there's going to be a _%(DATE)s in the tag name.

            e.g. B2G_1_3
            """
        return b2g_branch_config["tag_name"].split("_%")[0]

    def hg_tag(self, repo_name, b2g_branch_config):
        """ Attempt to tag and push gecko.  This assumes the trees are open.

            On failure, throw a VCSException.
            """
        hg = self.query_exe("hg", return_type="list")
        dirs = self.query_abs_dirs()
        hg_dir = os.path.join(dirs["abs_work_dir"], repo_name)
        tag_name = self.query_tag_name(b2g_branch_config)
        short_tag_name = self.query_short_tag_name(b2g_branch_config)
        push_url = self.query_repo_push_url(repo_name)
        cmd = hg + ["tag", tag_name, "-m",
                    "tagging %s for mergeday. r=a=mergeday CLOSED TREE DONTBUILD" % short_tag_name,
                    ]
        if self.run_command(cmd, cwd=hg_dir, error_list=HgErrorList):
            raise VCSException("Can't tag %s with %s" % (repo_name, tag_name))
        # Debugging! Echo only for now.
        # cmd = hg + ["push", push_url]
        cmd = hg + ["push", push_url]
        if self.run_command(cmd, cwd=hg_dir, error_list=HgErrorList):
            self.run_command(hg + ["--config", "extensions.mq=",
                                   "strip", "--no-backup", "outgoing()"],
                             cwd=hg_dir)
            self.run_command(hg + ["up", "-C"],
                             cwd=hg_dir)
            self.run_command(hg + ["--config", "extensions.purge=",
                                   "purge", "--all"],
                             cwd=hg_dir)
            raise VCSException("Can't push to %s!" % push_url)

    def git_tag(self, hg_repo_name, gaia_git_revision, b2g_branch_config):
        """ Attempt to tag and push gaia.

            On failure, throw a VCSException.
            """
        git = self.query_exe("git", return_type="list")
        dirs = self.query_abs_dirs()
        tag_name = self.query_tag_name(b2g_branch_config)
        cmd = git + ['tag', tag_name]
        if gaia_git_revision is None:
            if self.run_command(
                git + ["checkout", b2g_branch_config["gaia_branch"]],
                cwd=dirs["abs_gaia_dir"],
                error_list=GitErrorList,
            ):
                raise VCSException("Can't checkout %s branch!" % b2g_branch_config["gaia_branch"])
        else:
            cmd.append(gaia_git_revision)
        if self.run_command(
            cmd,
            cwd=dirs["abs_gaia_dir"],
            error_list=GitErrorList,
        ):
            raise VCSException("Can't tag gaia for %s!" % hg_repo_name)
        if self.run_command(
            # Debugging! Echo only for now.
            git + ["push", "--tags"],
            # comment out for testing
            # git + ["push", "--tags"],
            cwd=dirs["abs_gaia_dir"],
            error_list=GitErrorList,
        ):
            raise VCSException("Can't push gaia tag for %s!" % hg_repo_name)

# Actions {{{1
    def clean_repos(self):
        """ We may end up with contaminated local repos at some point, but
            we don't want to have to clobber and reclone from scratch every
            time.

            This is an attempt to clean up the local repos without needing a
            clobber.
            """
        dirs = self.query_abs_dirs()
        hg = self.query_exe("hg", return_type="list")
        git = self.query_exe("git", return_type="list")
        hg_repos = self.query_gecko_repos()
        hg_strip_error_list = [{
            'substr': r'''abort: empty revision set''', 'level': INFO,
            'explanation': "Nothing to clean up; we're good!",
        }] + HgErrorList
        for repo_name in hg_repos:
            repo_path = os.path.join(dirs['abs_work_dir'], repo_name)
            if os.path.exists(repo_path):
                self.retry(
                    self.run_command,
                    args=(hg + ["--config", "extensions.mq=", "strip",
                          "--no-backup", "outgoing()"], ),
                    kwargs={
                        'cwd': repo_path,
                        'error_list': hg_strip_error_list,
                        'return_type': 'num_errors',
                        'success_codes': (0, 255),
                    },
                )
        if os.path.exists(dirs['abs_gaia_dir']):
            remote_tags = []
            output = self.get_output_from_command(
                git + ["ls-remote", "--tags"],
                cwd=dirs['abs_gaia_dir'],
            )
            for line in output.splitlines():
                if 'refs/tags' not in line:
                    continue
                remote_tags.append(line.split('/')[-1])
            output = self.get_output_from_command(
                git + ["tag", "-l"],
                cwd=dirs['abs_gaia_dir'],
            )
            local_tags = output.splitlines()
            for tag in local_tags:
                if tag not in remote_tags:
                    self.info("Found local tag %s that doesn't exist on remote!" % tag)
                    self.run_command(
                        git + ['tag', '-d', tag],
                        cwd=dirs["abs_gaia_dir"],
                    )
            # Verify
            output = self.get_output_from_command(
                git + ["tag", "-l"],
                cwd=dirs['abs_gaia_dir'],
            )
            local_tags = output.splitlines()
            if not set(local_tags).issubset(set(remote_tags)):
                self.fatal("Gaia still has tags that don't exist on remote!")
            else:
                self.info("Looks like we're good, local_tags is a subset of remote_tags.")

    def pull(self):
        """ Pull action.

            Builds an hg repo list-of-dicts and sends them to
            MercurialScript.pull().  Also pulls the gaia_url.

            We'll potentially run another pull in the push-loop action, but
            this action makes sure we have an up-to-date clone on disk to
            operate on.
            """
        dirs = self.query_abs_dirs()
        git = self.query_exe("git", return_type="list")
        hg_repos = []
        for repo_name in self.query_gecko_repos():
            b2g_branch_config = self.config['b2g_branches'][repo_name]
            hg_repos.append(self.query_repo_pull_config(repo_name, b2g_branch_config))
        self.debug("HG repos: %s" % pprint.pformat(hg_repos))
        super(B2GTag, self).pull(repos=hg_repos)
        if not os.path.exists(dirs['abs_gaia_dir']):
            self.run_command(
                git + ["clone", self.config["gaia_url"], "gaia"],
                cwd=dirs['abs_work_dir'],
                error_list=GitErrorList,
                halt_on_failure=True,
            )
        else:
            self.run_command(
                git + ["pull"],
                cwd=dirs['abs_gaia_dir'],
                error_list=GitErrorList,
                halt_on_failure=True,
            )

    def push_loop(self):
        """ Create the tag and push for each gecko+gaia pair.
            This sometimes requires a pull+rebase, hence the loop.
            """
        for repo_name in self.query_gecko_repos():
            b2g_branch_config = self.config['b2g_branches'][repo_name]
            repo_config = self.query_repo_pull_config(repo_name, b2g_branch_config)
            super(B2GTag, self).pull(repos=[repo_config])
            try:
                # We'll get None (tag tip-of-branch) or a git revision or a
                # VCSException here.
                gaia_git_revision = self.query_gaia_git_revision(repo_name, b2g_branch_config)
            except VCSException, e:
                self.add_failure(repo_name, message=str(e))
                continue
            if self.retry(
                self.hg_tag,
                retry_exceptions=(VCSException, ),
                args=(repo_name, b2g_branch_config),
            ):
                self.add_failure(repo_name, message=str(e))
                continue
            if self.retry(
                self.git_tag,
                retry_exceptions=(VCSException, ),
                args=(repo_name, gaia_git_revision, b2g_branch_config),
            ):
                self.add_failure(repo_name, message=str(e))
                continue


# __main__ {{{1
if __name__ == '__main__':
    b2g_tag = B2GTag()
    b2g_tag.run_and_exit()



#!/usr/bin/env python
# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****

import copy
import getpass
import os
import pprint
import sys
import time
import socket

# load modules from parent dir
sys.path.insert(1, os.path.dirname(sys.path[0]))

from mozharness.mozilla.buildbot import BuildbotMixin
from mozharness.base.config import parse_config_file
from mozharness.base.log import INFO, FATAL
from mozharness.base.vcs.vcsbase import MercurialScript
from mozharness.mozilla.blob_upload import BlobUploadMixin, blobupload_config_options
from mozharness.mozilla.testing.mozpool import MozpoolMixin
from mozharness.mozilla.testing.device import SUTDeviceMozdeviceMixin
from mozharness.mozilla.testing.testbase import TestingMixin, testing_config_options
from mozharness.mozilla.testing.talos import TalosOutputParser, TalosErrorList

SUITE_CATEGORIES = ['talos', ]


class PandaTalosTest(TestingMixin, MercurialScript, BlobUploadMixin, MozpoolMixin, BuildbotMixin, SUTDeviceMozdeviceMixin):
    test_suites = SUITE_CATEGORIES
    config_options = [
        [["--mozpool-api-url"], {
            "dest": "mozpool_api_url",
            "help": "Override mozpool api url",
        }],
        [["--mozpool-device"], {
            "dest": "mozpool_device",
            "help": "Set Panda device to run tests on",
        }],
        [["--mozpool-assignee"], {
            "dest": "mozpool_assignee",
            "help": "Set mozpool assignee (requestor name, free-form)",
        }],
        [["--total-chunks"], {
            "action": "store",
            "dest": "total_chunks",
            "help": "Number of total chunks",
        }],
        [["--this-chunk"], {
            "action": "store",
            "dest": "this_chunk",
            "help": "Number of this chunk",
        }],
        [["--extra-args"], {
            "action": "store",
            "dest": "extra_args",
            "help": "Extra arguments",
        }],
        [['--talos-suite', ], {
            "action": "extend",
            "dest": "specified_talos_suites",
            "type": "string",
            "help": "Specify which talos suite to run. "
                    "Suites are defined in the config file\n."
                    "Examples: 'remote-tsvgx', 'remote-tp4m'",
        }],
        [["--branch-name"], {
            "action": "store",
            "dest": "talos_branch",
            "help": "Graphserver branch to report to"
        }],
        [['--run-all-suites', ], {
            "action": "store_true",
            "dest": "run_all_suites",
            "default": False,
            "help": "This will run all suites that are specified "
                    "in the config file. You do not need to specify "
                    "any other suites. Beware, this may take a while ;)"}
         ],
    ] + copy.deepcopy(testing_config_options) + \
        copy.deepcopy(blobupload_config_options)

    error_list = []
    mozpool_handler = None

    virtualenv_modules = [
        'mozpoolclient',
        'mozcrash'
    ]

    def __init__(self, require_config_file=False):
        super(PandaTalosTest, self).__init__(
            config_options=self.config_options,
            all_actions=['clobber',
                         'read-buildbot-config',
                         'download-and-extract',
                         'create-virtualenv',
                         'request-device',
                         'run-test',
                         'close-request'],
            default_actions=['clobber',
                             'read-buildbot-config',
                             'download-and-extract',
                             'create-virtualenv',
                             'request-device',
                             'run-test',
                             'close-request'],
            require_config_file=require_config_file,
            config={'virtualenv_modules': self.virtualenv_modules})

        self.mozpool_assignee = self.config.get('mozpool_assignee', getpass.getuser())
        self.request_url = None
        self.test_url = self.config.get("test_url")
        self.mozpool_device = self.config.get("mozpool_device")
        self.talos_branch = self.config.get("talos_branch")

        self.read_buildbot_config()
        self.revision = self.config.get('revision',
                                        self.buildbot_config.get('properties')["revision"])
        self.repo_path = self.config.get('repo_path',
                                         self.buildbot_config.get('properties')["repo_path"])
        self.talos_json_url = (self.config.get("talos_json_url") % (self.repo_path, self.revision))
        self.talos_json_config = None

    def postflight_read_buildbot_config(self):
        super(PandaTalosTest, self).postflight_read_buildbot_config()
        self.mozpool_device = self.config.get('mozpool_device', self.buildbot_config.get('properties')["slavename"])
        dirs = self.query_abs_dirs()
        #touch the shutdown file
        shutdown_file = os.path.join(dirs['shutdown_dir'], 'shutdown.stamp')
        try:
            self.info("*** Touching the shutdown file **")
            open(shutdown_file, 'w').close()
        except Exception, e:
            self.warning("We failed to create the shutdown file: str(%s)" % str(e))

    def query_talos_json_config(self):
        if self.talos_json_config:
            return self.talos_json_config

        dirs = self.query_abs_dirs()
        self.talos_json = self.download_file(self.talos_json_url,
                                             parent_dir=dirs['abs_talosdata_dir'],
                                             error_level=FATAL)
        self.talos_json_config = parse_config_file(self.talos_json)
        self.info(pprint.pformat(self.talos_json_config))

        return self.talos_json_config

    def request_device(self):
        self.retrieve_android_device(b2gbase="")
        env = self.query_env()
        cmd = [self.query_exe('python'), self.config.get("verify_path")]
        if self.run_command(cmd, env=env):
            self.critical("Preparing to abort run due to failed verify check.")
            self.close_request()
            self.fatal("Dying due to failing verification")
        else:
            self.info("Successfully verified the device")

    def _sut_prep_steps(self):
        device_time = self.set_device_epoch_time()
        self.info("Current time on device: %s - %s" %
                  (device_time, time.strftime("%x %H:%M:%S", time.gmtime(float(device_time)))))

    def preflight_talos(self, suite_category, suites):
        """preflight perf config etc"""
        env = self.query_env(partial_env={'DM_TRANS': "sut", 'TEST_DEVICE': self.mozpool_device})
        self.info("Running preflight...")
        preflight_category = "preflight_" + str(suite_category)
        dirs = self.query_abs_dirs()
        abs_base_cmd = self._query_abs_base_cmd(preflight_category)
        cmd = abs_base_cmd[:]
        replace_dict = {}
        for suite in suites:
            for arg in suites[suite]:
                cmd.append(arg % replace_dict)
        self._install_app()
        self.run_command(cmd, dirs['abs_talosdatatalos_dir'], env=env, halt_on_failure=True, fatal_exit_code=suites.get('fatal_exit_code', 3))

    def _run_category_suites(self, suite_category, preflight_run_method=None):
        """run suite(s) to a specific category"""

        env = self.query_env(partial_env={'DM_TRANS': "sut", 'TEST_DEVICE': self.mozpool_device})
        self.info("Running tests...")

        suites = self._query_specified_suites(suite_category)

        if preflight_run_method:
            preflight_run_method(suite_category, suites)
        if suites:
            self.info('#### Running %s suites' % suite_category)
            for suite in suites:
                dirs = self.query_abs_dirs()
                abs_base_cmd = self._query_abs_base_cmd(suite_category)
                cmd = abs_base_cmd[:]
                tbpl_status, log_level = None, None
                c = self.config
                if c.get('minidump_stackwalk_path'):
                    env['MINIDUMP_STACKWALK'] = c['minidump_stackwalk_path']
                env['MOZ_UPLOAD_DIR'] = self.query_abs_dirs()['abs_blob_upload_dir']
                env['MINIDUMP_SAVE_PATH'] = self.query_abs_dirs()['abs_blob_upload_dir']
                if not os.path.isdir(env['MOZ_UPLOAD_DIR']):
                    self.mkdir_p(env['MOZ_UPLOAD_DIR'])
                env = self.query_env(partial_env=env, log_level=INFO)

                parser = TalosOutputParser(config=self.config, log_obj=self.log_obj,
                                           error_list=TalosErrorList)

                if parser.minidump_output:
                    self.info("Looking at the minidump files for debugging purposes...")
                    for item in parser.minidump_output:
                        self.run_command(["ls", "-l", item])
                return_code = self.run_command(cmd, dirs['abs_talosdatatalos_dir'], env=env, output_parser=parser)
                if return_code != 0:
                    self.fatal("Failed talos " + str(cmd) + " command run.")

    def _query_specified_suites(self, category):
        # logic goes: if at least one '--{category}-suite' was given,
        # then run only that(those) given suite(s). Elif no suites were
        # specified and the --run-all-suites flag was given,
        # run all {category} suites. Anything else, run no suites.
        c = self.config
        all_suites = c.get('all_%s_suites' % (category))
        specified_suites = c.get('specified_%s_suites' % (category))  # list

        suites = None

        if specified_suites:
            if 'all' in specified_suites:
                # useful if you want a quick way of saying run all suites
                # of a specific category.
                suites = all_suites
            else:
                # suites gets a dict of everything from all_suites where a key
                # is also in specified_suites
                suites = dict((key, all_suites.get(key)) for key in
                              specified_suites if key in all_suites.keys())
        else:
            if c.get('run_all_suites'):  # needed if you dont specify any suites
                suites = all_suites

        return suites

    def run_test(self):
       # do we need to set the device time? command doesn't work anyways
       # self._sut_prep_steps()
        env = self.query_env()
        env["DM_TRANS"] = "sut"
        env["TEST_DEVICE"] = self.mozpool_device
        self.info("Running tests...")

        for category in SUITE_CATEGORIES:
            self._run_category_suites(category, preflight_run_method=self.preflight_talos)

    def _install_app(self):
        dirs = self.query_abs_dirs()
        cmd = ['python', self.config.get("install_app_path"), self.device_ip, os.path.join(dirs['abs_talosdata_dir'], self.filename_apk), self.app_name]
        self.run_command(cmd, dirs['abs_talosdata_dir'], halt_on_failure=True, fatal_exit_code=3)

    def query_abs_dirs(self):
        if self.abs_dirs:
            return self.abs_dirs
        abs_dirs = super(PandaTalosTest, self).query_abs_dirs()
        dirs = {}
        dirs['shutdown_dir'] = abs_dirs['abs_work_dir'].rsplit("/", 2)[0]
        dirs['abs_fennec_dir'] = os.path.join(
            dirs['shutdown_dir'], 'talos-data/fennec')
        dirs['abs_talosdata_dir'] = os.path.join(
            dirs['shutdown_dir'], 'talos-data')
        dirs['abs_symbols_dir'] = os.path.join(
            dirs['abs_talosdata_dir'], 'symbols')
        dirs['abs_talosdatatalos_dir'] = os.path.join(
            dirs['shutdown_dir'], 'talos-data/talos')
        dirs['abs_talosbuild_dir'] = os.path.join(
            dirs['shutdown_dir'], 'talos-data/build')
        dirs['abs_talos_dir'] = dirs['abs_talosdatatalos_dir']
        dirs['abs_preflight_talos_dir'] = dirs['abs_talosdatatalos_dir']
        dirs['abs_blob_upload_dir'] = os.path.join(abs_dirs['abs_work_dir'], 'blobber_upload_dir')
        for key in dirs.keys():
            if key not in abs_dirs:
                abs_dirs[key] = dirs[key]
        self.abs_dirs = abs_dirs
        return self.abs_dirs

    def download_and_extract(self):
        dirs = self.query_abs_dirs()

        #mkdir talos in  /builds/panda-0052/test/../talos-data
        fennec_ids_url = self.installer_url.rsplit("/", 1)[0] + "/fennec_ids.txt"
        self.mkdir_p(dirs['abs_talosbuild_dir'])
        robocop_url = self.installer_url.rsplit("/", 1)[0] + "/robocop.apk"
        self.mkdir_p(dirs['abs_talosdatatalos_dir'])

        #download and extract apk to /builds/panda-0nnn/talos-data
        self.rmtree(dirs['abs_talosdata_dir'])
        self.mkdir_p(dirs['abs_talosdata_dir'])
        self.mkdir_p(dirs['abs_symbols_dir'])
        self.download_unzip(self.installer_url,
                             dirs['abs_fennec_dir'])
        #this is ugly but you can't specify a file in download_unzip to extract the file to, by default it's the abs_work_dir
        #should think of a better way
        self.download_file(self.installer_url,
                           parent_dir=dirs['abs_talosdata_dir'],
                           error_level=FATAL)

        #download and extract fennec_ids.txt to /builds/panda-0nnn/talos-data
        self.download_file(fennec_ids_url, file_name='fennec_ids.txt',
                           parent_dir=dirs['abs_talosdata_dir'],
                           error_level=FATAL)
        #download and extract robocop.apk to /builds/panda-0nnn/talos-data/build
        self.download_file(robocop_url, file_name='robocop.apk',
                           parent_dir=dirs['abs_talosbuild_dir'],
                           error_level=FATAL)
        self.symbols_url = self.query_symbols_url()

        self.download_unzip(self.symbols_url,
                             dirs['abs_symbols_dir'])

        self.download_unzip(self.config['retry_url'],
                             dirs['abs_talosdata_dir'])

        taloscode = self.config.get("talos_from_code_url")

        talos_from_code_url = (taloscode % (self.repo_path, self.revision))

        self.download_file(talos_from_code_url, file_name='talos_from_code.py',
                           parent_dir=dirs['abs_talosdata_dir'],
                           error_level=FATAL)

        talos_base_cmd = ['python']
        talos_code_path = (os.path.join(dirs['abs_talosdata_dir'], "talos_from_code.py"))
        talos_zip_path = (os.path.join(dirs['abs_talosdata_dir'], "talos.zip"))
        talos_base_cmd.append(talos_code_path)
        talos_base_cmd.append("--talos-json-url")
        talos_base_cmd.append(self.talos_json_url)
        env = self.query_env()
        self.run_command(talos_base_cmd, dirs['abs_talosdata_dir'], env=env, halt_on_failure=True, fatal_exit_code=3)
        unzip = self.query_exe("unzip")
        unzip_cmd = [unzip, '-q', '-o',  talos_zip_path]
        self.run_command(unzip_cmd, cwd=dirs['abs_talosdata_dir'], halt_on_failure=True, fatal_exit_code=3)

    def _query_abs_base_cmd(self, suite_category):
        dirs = self.query_abs_dirs()
        options = []
        run_file = self.config['run_file_names'][suite_category]
        base_cmd = ['python', '-u']
        base_cmd.append(os.path.join((dirs["abs_%s_dir" % suite_category]), run_file))
        self.device_ip = socket.gethostbyname(self.mozpool_device)
        hostnumber = int(self.mozpool_device.split('-')[1])
        http_port = '30%03i' % hostnumber
        ssl_port = '31%03i' % hostnumber
        #get filename from installer_url
        self.filename_apk = self.installer_url.split('/')[-1]
        #find appname from package-name.txt - assumes download-and-extract has completed successfully
        apk_dir = self.abs_dirs['abs_work_dir']
        self.apk_path = os.path.join(apk_dir, self.filename_apk)
        unzip = self.query_exe("unzip")
        package_path = os.path.join(dirs['abs_fennec_dir'], 'package-name.txt')
        unzip_cmd = [unzip, '-q', '-o',  self.apk_path]
        self.run_command(unzip_cmd, cwd=dirs['abs_fennec_dir'], halt_on_failure=True, fatal_exit_code=3)
        self.app_name = str(self.read_from_file(package_path, verbose=True)).rstrip()

        str_format_values = {
            'device_ip': self.device_ip,
            'hostname': self.mozpool_device,
            'http_port': http_port,
            'ssl_port':  ssl_port,
            'app_name':  self.app_name,
            'talos_branch':  self.talos_branch,
        }
        talos_json_config = self.query_talos_json_config()
        if talos_json_config.get('extra_options') and \
           talos_json_config['extra_options'].get('android'):
            for option in self.talos_json_config['extra_options']['android']:
                options.append(option % {
                    'apk_path': self.apk_path })

        if self.config['%s_options' % suite_category]:
            for option in self.config['%s_options' % suite_category]:
                options.append(option % str_format_values)
            for url in self.config.get('datazilla_urls', []):
                options.extend(['--datazilla-url', url])
            # add datazilla authfile
            authfile = self.config.get('datazilla_authfile')
            if authfile:
                options.extend(['--authfile', authfile])
            abs_base_cmd = base_cmd + options
            return abs_base_cmd
        else:
            self.warning("Suite options for %s could not be determined."
                         "\nIf you meant to have options for this suite, "
                         "please make sure they are specified in your "
                         "config under %s_options" %
                         (suite_category, suite_category))

    ###### helper methods
    def _pre_config_lock(self, rw_config):
        super(PandaTalosTest, self)._pre_config_lock(rw_config)
        c = self.config
        if not c.get('run_all_suites'):
            return  # configs are valid
        for category in SUITE_CATEGORIES:
            specific_suites = c.get('specified_%s_suites' % (category))
            if specific_suites:
                if specific_suites != 'all':
                    self.fatal("Config options are not valid. Please ensure"
                               " that if the '--run-all-suites' flag was enabled,"
                               " then do not specify to run only specific suites "
                               "like:\n '--mochitest-suite browser-chrome'")

    def _post_fatal(self, message=None, exit_code=None):
        """ After we call fatal(), run this method before exiting.
            """
        self.close_request()

    def close_request(self):
        if self.request_url:
            mph = self.query_mozpool_handler(self.mozpool_device)
            mph.close_request(self.request_url)
            self.info("Request '%s' deleted on cleanup" % self.request_url)
            self.request_url = None
        else:
            self.info("request_url doesn't exist. Already closed?")

    def _build_arg(self, option, value):
        """
        Build a command line argument
        """
        if not value:
            return []
        return [str(option), str(value)]

if __name__ == '__main__':
    pandaTalosTest = PandaTalosTest()
    pandaTalosTest.run_and_exit()



# This is a template config file for b2g emulator unittest testing
import platform

HG_SHARE_BASE_DIR = "/builds/hg-shared"

if platform.system().lower() == 'darwin':
    xre_url = "https://api.pub.build.mozilla.org/tooltool/sha512/4d8d7a37d90c34a2a2fda3066a8fe85c189b183d05389cb957fc6fed31f10a6924e50c1b84488ff61c015293803f58a3aed5d4819374d04c8e0ee2b9e3997278"
else:
    xre_url = "https://api.pub.build.mozilla.org/tooltool/sha512/dc9503b21c87b5a469118746f99e4f41d73888972ce735fa10a80f6d218086da0e3da525d9a4cd8e4ea497ec199fef720e4a525873d77a1af304ac505e076462"

config = {
    # mozharness script options
    "xre_url": xre_url,

    "vcs_share_base": HG_SHARE_BASE_DIR,
    "exes": {
        'python': '/tools/buildbot/bin/python',
        'virtualenv': ['/tools/buildbot/bin/python', '/tools/misc-python/virtualenv.py'],
        'tooltool.py': "/tools/tooltool.py",
    },

    "find_links": [
        "http://pypi.pvt.build.mozilla.org/pub",
        "http://pypi.pub.build.mozilla.org/pub",
    ],
    "pip_index": False,

    "buildbot_json_path": "buildprops.json",

    "default_actions": [
        'clobber',
        'read-buildbot-config',
        'pull',
        'download-and-extract',
        'create-virtualenv',
        'install',
        'run-tests',
    ],
    "vcs_output_timeout": 1760,
}
